from django.urls import path

from django_spire.help_desk.views import form_views

app_name = 'form'

urlpatterns = [
    path('create/', form_views.ticket_create_form_view, name='create'),
    path('<int:pk>/update/', form_views.ticket_update_form_view, name='update'),
]



from __future__ import annotations

import pytest

from dataclasses import fields

from django.test import TestCase

from django_spire.theme.enums import ThemeFamily, ThemeMode
from django_spire.theme.models import Theme


class ThemeModelTests(TestCase):
    def test_dataclass_fields(self) -> None:
        theme_fields = {f.name: f.type for f in fields(Theme)}

        self.assertEqual(set(theme_fields.keys()), {'family', 'mode'})
        self.assertEqual(theme_fields['family'], 'ThemeFamily')
        self.assertEqual(theme_fields['mode'], 'ThemeMode')

    def test_class_variables(self) -> None:
        self.assertEqual(Theme.DEFAULT_FAMILY, ThemeFamily.DEFAULT)
        self.assertEqual(Theme.DEFAULT_MODE, ThemeMode.LIGHT)
        self.assertEqual(Theme.SEPARATOR, '-')

        for family in ThemeFamily:
            self.assertIn(family, Theme.FAMILY_DISPLAY_NAMES)

    def test_theme_initialization_with_enums(self) -> None:
        theme = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.DARK)
        self.assertEqual(theme.family, ThemeFamily.GRUVBOX)
        self.assertEqual(theme.mode, ThemeMode.DARK)

    def test_theme_initialization_with_strings(self) -> None:
        theme = Theme(family='gruvbox', mode='dark')
        self.assertEqual(theme.family, ThemeFamily.GRUVBOX)
        self.assertEqual(theme.mode, ThemeMode.DARK)

    def test_theme_initialization_invalid_family(self) -> None:
        with pytest.raises(ValueError) as ctx:
            Theme(family='invalid-family', mode='dark')
        self.assertIn('Invalid theme family', str(ctx.value))

    def test_theme_initialization_invalid_mode(self) -> None:
        with pytest.raises(ValueError) as ctx:
            Theme(family='gruvbox', mode='invalid-mode')
        self.assertIn('Invalid theme mode', str(ctx.value))

    def test_from_string_valid(self) -> None:
        cases = [
            ('default-light', ThemeFamily.DEFAULT, ThemeMode.LIGHT),
            ('gruvbox-dark', ThemeFamily.GRUVBOX, ThemeMode.DARK),
            ('one-dark-light', ThemeFamily.ONE_DARK, ThemeMode.LIGHT),
            ('tokyo-night-dark', ThemeFamily.TOKYO_NIGHT, ThemeMode.DARK),
        ]

        for string, family, mode in cases:
            with self.subTest(string=string):
                theme = Theme.from_string(string)
                self.assertEqual(theme.family, family)
                self.assertEqual(theme.mode, mode)

    def test_from_string_empty_with_default(self) -> None:
        default = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.DARK)
        theme = Theme.from_string('', default=default)
        self.assertEqual(theme, default)

    def test_from_string_empty_without_default(self) -> None:
        theme = Theme.from_string('')
        self.assertEqual(theme.family, Theme.DEFAULT_FAMILY)
        self.assertEqual(theme.mode, Theme.DEFAULT_MODE)

    def test_from_string_invalid_with_default(self) -> None:
        default = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.DARK)
        theme = Theme.from_string('invalid', default=default)
        self.assertEqual(theme, default)

    def test_from_string_invalid_without_default(self) -> None:
        with pytest.raises(ValueError):
            Theme.from_string('invalid')

    def test_get_available(self) -> None:
        available = Theme.get_available()

        count = len(ThemeFamily) * len(ThemeMode)
        self.assertEqual(len(available), count)

        for family in ThemeFamily:
            for mode in ThemeMode:
                theme = Theme(family=family, mode=mode)
                self.assertIn(theme, available)

    def test_get_default(self) -> None:
        default = Theme.get_default()
        self.assertEqual(default.family, Theme.DEFAULT_FAMILY)
        self.assertEqual(default.mode, Theme.DEFAULT_MODE)

    def test_display_property(self) -> None:
        theme = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.DARK)
        self.assertEqual(theme.display, 'Gruvbox - Dark')

        theme = Theme(family=ThemeFamily.ONE_DARK, mode=ThemeMode.LIGHT)
        self.assertEqual(theme.display, 'One Dark Pro - Light')

    def test_family_display_property(self) -> None:
        theme = Theme(family=ThemeFamily.ONE_DARK, mode=ThemeMode.DARK)
        self.assertEqual(theme.family_display, 'One Dark Pro')

    def test_is_dark_property(self) -> None:
        dark = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.DARK)
        self.assertTrue(dark.is_dark)

        light = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.LIGHT)
        self.assertFalse(light.is_dark)

    def test_stylesheet_property(self) -> None:
        cases = [
            (ThemeFamily.DEFAULT, ThemeMode.LIGHT, 'django_spire/css/themes/default/app-light.css'),
            (ThemeFamily.GRUVBOX, ThemeMode.DARK, 'django_spire/css/themes/gruvbox/app-dark.css'),
            (ThemeFamily.ONE_DARK, ThemeMode.LIGHT, 'django_spire/css/themes/one-dark/app-light.css'),
            (ThemeFamily.TOKYO_NIGHT, ThemeMode.DARK, 'django_spire/css/themes/tokyo-night/app-dark.css'),
        ]

        for family, mode, path in cases:
            with self.subTest(family=family, mode=mode):
                theme = Theme(family=family, mode=mode)
                self.assertEqual(theme.stylesheet, path)

    def test_value_property(self) -> None:
        cases = [
            (ThemeFamily.DEFAULT, ThemeMode.LIGHT, 'default-light'),
            (ThemeFamily.GRUVBOX, ThemeMode.DARK, 'gruvbox-dark'),
            (ThemeFamily.ONE_DARK, ThemeMode.LIGHT, 'one-dark-light'),
            (ThemeFamily.TOKYO_NIGHT, ThemeMode.DARK, 'tokyo-night-dark'),
        ]

        for family, mode, value in cases:
            with self.subTest(family=family, mode=mode):
                theme = Theme(family=family, mode=mode)
                self.assertEqual(theme.value, value)

    def test_to_dict(self) -> None:
        theme = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.DARK)
        result = theme.to_dict()

        keys = {
            'display',
            'family',
            'family_display',
            'full',
            'is_dark',
            'mode',
            'stylesheet'
        }

        self.assertEqual(set(result.keys()), keys)

        self.assertEqual(result['display'], 'Gruvbox - Dark')
        self.assertEqual(result['family'], 'gruvbox')
        self.assertEqual(result['family_display'], 'Gruvbox')
        self.assertEqual(result['full'], 'gruvbox-dark')
        self.assertTrue(result['is_dark'])
        self.assertEqual(result['mode'], 'dark')
        self.assertEqual(result['stylesheet'], 'django_spire/css/themes/gruvbox/app-dark.css')

    def test_theme_immutability(self) -> None:
        theme = Theme(family=ThemeFamily.GRUVBOX, mode=ThemeMode.DARK)

        with pytest.raises(AttributeError):
            theme.family = ThemeFamily.DEFAULT

        with pytest.raises(AttributeError):
            theme.mode = ThemeMode.LIGHT



from __future__ import annotations


def find_user_list_from_content_type(app_label, model_name):
    from django.contrib.contenttypes.models import ContentType
    from django.contrib.auth.models import Permission, Group, User

    content_type = ContentType.objects.get(app_label=app_label, model=model_name)
    permission = Permission.objects.get(content_type=content_type, codename=f'view_{model_name}')
    groups = Group.objects.filter(permissions=permission)

    return User.objects.filter(groups__in=groups).distinct()


def generate_comment_user_list_data(user_list):
    user_data = []

    for user in user_list:
        user_data.append({
            'full_name': user.get_full_name().replace(' ', '_'),
            'id': user.pk,
        })

    return user_data


def parse_user_id_to_int_list(user_id_str):
    from django.contrib.auth.models import User

    user_id_list =  [int(user_id) for user_id in user_id_str.split(',')]
    return User.objects.filter(id__in=user_id_list)


def process_comment_notifications(user_list, comment_information, related_obj, user_commenting):
    from django_spire.notification.models import Notification
    from django.contrib.sites.models import Site

    for user in user_list:
        if user != user_commenting:
            Notification.create(
                email=user.email,
                title='New Comment',
                body=f'You were tagged in a new comment! {user_commenting.get_full_name()} wrote "{comment_information}" on {related_obj}.',
                url=Site.objects.get_current()
            )



from __future__ import annotations

from django.test import TestCase

from django_spire.theme.enums import ThemeFamily, ThemeMode


class ThemeEnumTests(TestCase):
    def test_theme_families_match_filesystem(self) -> None:
        expected = {
            'ayu',
            'catppuccin',
            'default',
            'gruvbox',
            'material',
            'nord',
            'one-dark',
            'palenight',
            'rose-pine',
            'tokyo-night'
        }

        actual = {family.value for family in ThemeFamily}
        self.assertEqual(actual, expected)

    def test_theme_modes(self) -> None:
        expected = {'dark', 'light'}
        actual = {mode.value for mode in ThemeMode}
        self.assertEqual(actual, expected)



from __future__ import annotations

import json

from django.conf import settings
from django.contrib.sites.models import Site
from django.db.models import Prefetch
from django.urls import reverse

from django_spire.contrib.service import BaseDjangoModelService

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django_spire.knowledge.collection.models import Collection


class CollectionTransformationService(BaseDjangoModelService['Collection']):
    obj: Collection

    def to_hierarchy_json(self, request: WSGIRequest, parent_id: int | None = None) -> str:
        collections = (
            self.obj_class.objects
            .active()
            .by_parent_id(parent_id=parent_id)
            .request_user_has_access(request)
            .select_related('parent')
        )

        entry_queryset = (
            collections.model._meta.fields_map.get('entry')
            .related_model
            .objects
            .active()
            .has_current_version()
            .user_has_access(user=request.user)
            .select_related('current_version__author')
            .order_by('order')
        )

        collections = list(
            collections.prefetch_related(Prefetch('entries', queryset=entry_queryset))
            .active()
            .order_by('order')
        )

        collection_map = {}
        for collection in collections:
            collection_map[collection.pk] = collection.services.transformation.to_dict()

        tree = []
        for collection in collections:
            if collection.parent_id and collection.parent_id in collection_map:
                collection_map[collection.parent_id]['children'].append(
                    collection_map[collection.pk]
                )
            else:
                tree.append(collection_map[collection.pk])

        return json.dumps(tree)

    def to_dict(self):
        site = Site.objects.get_current() if not settings.DEBUG else ''
        entries = self.obj.entries.all()

        return {
            'id': self.obj.pk,
            'name': self.obj.name,
            'description': self.obj.description,
            'children': [],
            'entries': (
                entries.model.services.transformation
                .queryset_to_navigation_list(queryset=entries)
            ),
            'delete_url': f'''
                {site}{
                    reverse(
                        'django_spire:knowledge:collection:page:delete',
                        kwargs={'pk': self.obj.pk},
                    )
                }
            ''',
            'edit_url': f'''
                {site}{
                    reverse(
                        'django_spire:knowledge:collection:form:update',
                        kwargs={'pk': self.obj.pk},
                    )
                }
            ''',
            'create_entry_url': f'''
                {site}{
                    reverse(
                        'django_spire:knowledge:entry:form:create',
                        kwargs={'collection_pk': self.obj.pk},
                    )
                }
            ''',
            'import_entry_url': f'''
                {site}{
                    reverse(
                        'django_spire:knowledge:entry:form:import',
                        kwargs={'collection_pk': self.obj.pk},
                    )
                }
            ''',
        }



import inspect
from inspect import Signature
from typing import Callable, Any, Sequence


def get_object_from_module_string(module_string: str) -> Any:
    try:
        module_string, object_name = module_string.rsplit('.', 1)
        module = __import__(module_string, fromlist=[object_name])
    except ImportError:
        raise ImportError(f'Could not import module: {module_string}')

    return getattr(module, object_name)

def get_callable_from_module_string_and_validate_arguments(
        module_string: str,
        valid_args: Sequence[str],
) -> Callable:
    callable_ = get_object_from_module_string(module_string)

    if not callable(callable_):
        raise TypeError(f'Object {module_string} is not callable')

    sig = inspect.signature(callable_)
    params = sig.parameters

    for name in valid_args:
        if name not in params:
            message = f'{callable_.__qualname__} is missing required argument: {name}'
            raise TypeError(message)

    return callable_


from __future__ import annotations

from dataclasses import asdict, dataclass


@dataclass
class AppConfiguration:
    """
    Django app configuration and labeling.

    Example:
        app_config_class_name: 'EmployeeSkillConfig'
        app_name_component: 'skill'
        django_label: 'employee_skill'
        permission_prefix: 'employee_skill'
        db_table_name: 'employee_skill'

    """

    app_config_class_name: str
    app_name_component: str
    db_table_name: str
    django_label: str
    permission_prefix: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> AppConfiguration:
        immediate_parent = components[-2] if len(components) > 2 else None
        parent_parts = components[1:-1] if len(components) > 1 else []

        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        default_model_name = ''.join(
            word.title()
            for word in app_name.split('_')
        )

        model_name = (
            user_inputs.get('model_name', default_model_name)
            if user_inputs
            else default_model_name
        )

        default_label = (
            immediate_parent.lower() + '_' + app_name.lower()
            if immediate_parent
            else app_name.lower()
        )

        app_label = (
            user_inputs.get('app_label', default_label)
            if user_inputs
            else default_label
        )

        default_db_table = (
            '_'.join(parent_parts).lower() + '_' + app_name.lower()
            if parent_parts
            else app_name.lower()
        )

        db_table = (
            user_inputs.get('db_table_name', default_db_table)
            if user_inputs
            else default_db_table
        )

        inherit_permissions = (
            user_inputs.get('inherit_permissions', False)
            if user_inputs
            else False
        )

        if inherit_permissions and user_inputs:
            permission_prefix = user_inputs.get('parent_permission_prefix', app_label)
        else:
            permission_prefix = app_label

        return cls(
            app_config_class_name=model_name + 'Config',
            app_name_component=app_name,
            db_table_name=db_table,
            django_label=app_label,
            permission_prefix=permission_prefix,
        )


@dataclass
class ContextVariables:
    """
    Template context variable names and django_glue keys.

    Example:
        glue_model_key: 'employee_skill'
        context_single_var: 'employee_skill'
        context_plural_var: 'employee_skills'

    """

    context_plural_var: str
    context_single_var: str
    glue_model_key: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> ContextVariables:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        return cls(
            context_plural_var=app_name.lower() + 's',
            context_single_var=app_name.lower(),
            glue_model_key=app_name.lower(),
        )


@dataclass
class DataClasses:
    """
    Data layer class names (seeders, querysets, forms).

    Example:
        seeder_class_name: 'EmployeeSkillSeeder'
        queryset_class_name: 'EmployeeSkillQuerySet'
        form_class_name: 'EmployeeSkillForm'

    """

    form_class_name: str
    queryset_class_name: str
    seeder_class_name: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> DataClasses:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        default_model_name = ''.join(
            word.title()
            for word in app_name.split('_')
        )

        model_name = (
            user_inputs.get('model_name', default_model_name)
            if user_inputs
            else default_model_name
        )

        return cls(
            form_class_name=model_name + 'Form',
            queryset_class_name=model_name + 'QuerySet',
            seeder_class_name=model_name + 'Seeder',
        )


@dataclass
class IntelligenceClasses:
    """
    LLM/AI related class names.

    Example:
        bot_class_name: 'EmployeeSkillBot'
        intel_class_name: 'EmployeeSkillIntel'

    """

    bot_class_name: str
    intel_class_name: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> IntelligenceClasses:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        default_model_name = ''.join(
            word.title()
            for word in app_name.split('_')
        )

        model_name = (
            user_inputs.get('model_name', default_model_name)
            if user_inputs
            else default_model_name
        )

        return cls(
            bot_class_name=model_name + 'Bot',
            intel_class_name=model_name + 'Intel',
        )


@dataclass
class ModelNames:
    """
    Model class and instance naming conventions.

    Example:
        model_class_name: 'EmployeeSkill'
        model_class_name_plural: 'EmployeeSkills'
        model_instance_name: 'employee_skill'
        model_instance_name_plural: 'employee_skills'
        model_verbose_name: 'Employee Skill'
        model_verbose_name_plural: 'Employee Skills'

    """

    model_class_name: str
    model_class_name_plural: str
    model_instance_name: str
    model_instance_name_plural: str
    model_verbose_name: str
    model_verbose_name_plural: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> ModelNames:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        default_model_name = ''.join(
            word.title()
            for word in app_name.split('_')
        )

        model_name = (
            user_inputs.get('model_name', default_model_name)
            if user_inputs
            else default_model_name
        )

        model_name_plural = (
            user_inputs.get('model_name_plural', model_name + 's')
            if user_inputs
            else model_name + 's'
        )

        default_verbose_name = ' '.join(
            word.title()
            for word in app_name.split('_')
        )

        verbose_name = (
            user_inputs.get('verbose_name', default_verbose_name)
            if user_inputs
            else default_verbose_name
        )

        verbose_name_plural = (
            user_inputs.get('verbose_name_plural', verbose_name + 's')
            if user_inputs
            else verbose_name + 's'
        )

        inherit_permissions = (
            user_inputs.get('inherit_permissions', False)
            if user_inputs
            else False
        )

        if inherit_permissions and user_inputs:
            model_instance_name = user_inputs.get('parent_model_instance_name', app_name.lower())
            model_instance_name_plural = model_instance_name + 's'
        else:
            model_instance_name = app_name.lower()
            model_instance_name_plural = app_name.lower() + 's'

        return cls(
            model_class_name=model_name,
            model_class_name_plural=model_name_plural,
            model_instance_name=model_instance_name,
            model_instance_name_plural=model_instance_name_plural,
            model_verbose_name=verbose_name,
            model_verbose_name_plural=verbose_name_plural,
        )


@dataclass
class ModelPermissions:
    """
    MODEL_PERMISSIONS configuration for apps.py.

    Example:
        permission_name: 'employee_skill'
        model_class_path: 'app.human_resource.employee.skill.models.EmployeeSkill'
        is_proxy_model: False

    """

    is_proxy_model: bool
    model_class_path: str
    permission_name: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> ModelPermissions:
        module = '.'.join(components)
        immediate_parent = components[-2] if len(components) > 2 else None

        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        default_model_name = ''.join(
            word.title()
            for word in app_name.split('_')
        )

        model_name = (
            user_inputs.get('model_name', default_model_name)
            if user_inputs
            else default_model_name
        )

        default_permission_name = (
            immediate_parent.lower() + '_' + app_name.lower()
            if immediate_parent
            else app_name.lower()
        )

        inherit_permissions = (
            user_inputs.get('inherit_permissions', False)
            if user_inputs
            else False
        )

        if inherit_permissions and user_inputs:
            permission_path = user_inputs.get('parent_model_path', f'{module}.models.{model_name}')
        else:
            permission_path = (
                user_inputs.get('model_permission_path', f'{module}.models.{model_name}')
                if user_inputs
                else f'{module}.models.{model_name}'
            )

        return cls(
            is_proxy_model=False,
            model_class_path=permission_path,
            permission_name=default_permission_name,
        )


@dataclass
class ModulePaths:
    """
    Module and path references for Python imports.

    Example:
        module: 'app.human_resource.employee.skill'
        module_path: 'app.human_resource.employee.skill'

    """

    module: str
    module_path: str

    @classmethod
    def build(cls, components: list[str], **_kwargs) -> ModulePaths:
        module = '.'.join(components)
        return cls(module=module, module_path=module)


@dataclass
class ParentReferences:
    """
    Parent app and model references.

    Example:
        parent_app_name: 'employee'
        parent_model_class_name: 'Employee'

    """

    parent_app_name: str
    parent_model_class_name: str

    @classmethod
    def build(cls, components: list[str], **_kwargs) -> ParentReferences:
        parent = components[-2] if len(components) > 1 else components[-1]

        return cls(
            parent_app_name=parent.lower(),
            parent_model_class_name=''.join(
                word.title()
                for word in parent.split('_')
            ),
        )


@dataclass
class PromptFunctions:
    """
    LLM prompt function names.

    Example:
        instruction_prompt_function_name: 'employee_skill_instruction_prompt'
        user_input_prompt_function_name: 'employee_skill_user_input_prompt'

    """

    instruction_prompt_function_name: str
    user_input_prompt_function_name: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> PromptFunctions:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        return cls(
            instruction_prompt_function_name=app_name.lower() + '_instruction_prompt',
            user_input_prompt_function_name=app_name.lower() + '_user_input_prompt',
        )


@dataclass
class ReplacementMapBuilder:
    """Aggregates all dataclasses and builds the final replacement map."""

    app_config: AppConfiguration
    context: ContextVariables
    data_classes: DataClasses
    intelligence: IntelligenceClasses
    model_names: ModelNames
    model_permissions: ModelPermissions
    module_paths: ModulePaths
    parent_refs: ParentReferences
    prompts: PromptFunctions
    services: ServiceClasses
    templates: TemplatePaths
    tests: TestClasses
    urls: URLPatterns
    views: ViewFunctions

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> ReplacementMapBuilder:
        return cls(
            app_config=AppConfiguration.build(components, user_inputs=user_inputs),
            context=ContextVariables.build(components, user_inputs=user_inputs),
            data_classes=DataClasses.build(components, user_inputs=user_inputs),
            intelligence=IntelligenceClasses.build(components, user_inputs=user_inputs),
            model_names=ModelNames.build(components, user_inputs=user_inputs),
            model_permissions=ModelPermissions.build(components, user_inputs=user_inputs),
            module_paths=ModulePaths.build(components, user_inputs=user_inputs),
            parent_refs=ParentReferences.build(components, user_inputs=user_inputs),
            prompts=PromptFunctions.build(components, user_inputs=user_inputs),
            services=ServiceClasses.build(components, user_inputs=user_inputs),
            templates=TemplatePaths.build(components, user_inputs=user_inputs),
            tests=TestClasses.build(components, user_inputs=user_inputs),
            urls=URLPatterns.build(components, user_inputs=user_inputs),
            views=ViewFunctions.build(components, user_inputs=user_inputs),
        )

    def to_dict(self) -> dict[str, str]:
        """Convert the builder to a flat dictionary for template replacement."""

        return {
            **asdict(self.app_config),
            **asdict(self.context),
            **asdict(self.data_classes),
            **asdict(self.intelligence),
            **asdict(self.model_names),
            **asdict(self.model_permissions),
            **asdict(self.module_paths),
            **asdict(self.parent_refs),
            **asdict(self.prompts),
            **asdict(self.services),
            **asdict(self.templates),
            **asdict(self.tests),
            **asdict(self.urls),
            **asdict(self.views),
        }


@dataclass
class ServiceClasses:
    """
    Service layer class names.

    Example:
        service_class_name: 'EmployeeSkillService'
        factory_service_class_name: 'EmployeeSkillFactoryService'
        intelligence_service_class_name: 'EmployeeSkillIntelligenceService'
        processor_service_class_name: 'EmployeeSkillProcessorService'
        transformation_service_class_name: 'EmployeeSkillTransformationService'

    """

    factory_service_class_name: str
    intelligence_service_class_name: str
    processor_service_class_name: str
    service_class_name: str
    transformation_service_class_name: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> ServiceClasses:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        default_model_name = ''.join(
            word.title()
            for word in app_name.split('_')
        )

        model_name = (
            user_inputs.get('model_name', default_model_name)
            if user_inputs
            else default_model_name
        )

        return cls(
            factory_service_class_name=model_name + 'FactoryService',
            intelligence_service_class_name=model_name + 'IntelligenceService',
            processor_service_class_name=model_name + 'ProcessorService',
            service_class_name=model_name + 'Service',
            transformation_service_class_name=model_name + 'TransformationService',
        )


@dataclass
class TemplatePaths:
    """
    Template file paths and names.

    Example:
        template_directory_path: 'employee/skill'
        detail_card_template_name: 'employee_skill_detail_card.html'
        form_card_template_name: 'employee_skill_form_card.html'
        list_card_template_name: 'employee_skill_list_card.html'
        item_template_name: 'employee_skill_item.html'
        form_template_name: 'employee_skill_form.html'
        detail_page_template_name: 'employee_skill_detail_page.html'
        form_page_template_name: 'employee_skill_form_page.html'
        list_page_template_name: 'employee_skill_list_page.html'

    """

    detail_card_template_name: str
    detail_page_template_name: str
    form_card_template_name: str
    form_page_template_name: str
    form_template_name: str
    item_template_name: str
    list_card_template_name: str
    list_page_template_name: str
    template_directory_path: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> TemplatePaths:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        parent_parts = components[1:-1] if len(components) > 1 else []

        template_path = (
            '/'.join(parent_parts).lower() + '/' + app_name.lower()
            if parent_parts
            else app_name.lower()
        )

        return cls(
            detail_card_template_name=app_name.lower() + '_detail_card',
            detail_page_template_name=app_name.lower() + '_detail_page',
            form_card_template_name=app_name.lower() + '_form_card',
            form_page_template_name=app_name.lower() + '_form_page',
            form_template_name=app_name.lower() + '_form',
            item_template_name=app_name.lower() + '_item',
            list_card_template_name=app_name.lower() + '_list_card',
            list_page_template_name=app_name.lower() + '_list_page',
            template_directory_path=template_path,
        )


@dataclass
class TestClasses:
    """
    Test case class names.

    Example:
        model_test_class_name: 'EmployeeSkillModelTestCase'
        bot_test_class_name: 'EmployeeSkillBotTestCase'
        service_test_class_name: 'EmployeeSkillServiceTestCase'
        factory_service_test_class_name: 'EmployeeSkillFactoryServiceTestCase'
        intelligence_service_test_class_name: 'EmployeeSkillIntelligenceServiceTestCase'
        processor_service_test_class_name: 'EmployeeSkillProcessorServiceTestCase'
        transformation_service_test_class_name: 'EmployeeSkillTransformationServiceTestCase'
        url_test_class_name: 'EmployeeSkillUrlTestCase'
        view_test_class_name: 'EmployeeSkillViewTestCase'

    """

    bot_test_class_name: str
    factory_service_test_class_name: str
    intelligence_service_test_class_name: str
    model_test_class_name: str
    processor_service_test_class_name: str
    service_test_class_name: str
    transformation_service_test_class_name: str
    url_test_class_name: str
    view_test_class_name: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> TestClasses:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        default_model_name = ''.join(
            word.title()
            for word in app_name.split('_')
        )

        model_name = (
            user_inputs.get('model_name', default_model_name)
            if user_inputs
            else default_model_name
        )

        return cls(
            bot_test_class_name=model_name + 'BotTestCase',
            factory_service_test_class_name=model_name + 'FactoryServiceTestCase',
            intelligence_service_test_class_name=model_name + 'IntelligenceServiceTestCase',
            model_test_class_name=model_name + 'ModelTestCase',
            processor_service_test_class_name=model_name + 'ProcessorServiceTestCase',
            service_test_class_name=model_name + 'ServiceTestCase',
            transformation_service_test_class_name=model_name + 'TransformationServiceTestCase',
            url_test_class_name=model_name + 'UrlTestCase',
            view_test_class_name=model_name + 'ViewTestCase',
        )


@dataclass
class URLPatterns:
    """
    URL routing and namespaces.

    Example:
        url_namespace: 'skill'
        url_reverse_path: 'employee:skill'
        url_reverse_parent_path: 'employee'

    """

    url_namespace: str
    url_reverse_parent_path: str
    url_reverse_path: str

    @classmethod
    def build(
        cls,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> URLPatterns:
        app_name = (
            user_inputs.get('app_name', components[-1])
            if user_inputs
            else components[-1]
        )

        parent_parts = components[1:-1] if len(components) > 1 else []

        if parent_parts:
            reverse_parent = ':'.join(parent_parts).lower()
            reverse_path = reverse_parent + ':' + app_name.lower()
        else:
            reverse_parent = ''
            reverse_path = app_name.lower()

        return cls(
            url_namespace=app_name.lower(),
            url_reverse_parent_path=reverse_parent,
            url_reverse_path=reverse_path,
        )


@dataclass
class ViewFunctions:
    """
    View function names.

    Example:
        list_page_view_name: 'list_page_view'
        detail_page_view_name: 'detail_page_view'
        create_form_view_name: 'create_form_view'
        update_form_view_name: 'update_form_view'
        delete_form_view_name: 'delete_form_view'
        create_modal_form_view_name: 'create_modal_form_view'
        update_modal_form_view_name: 'update_modal_form_view'
        delete_modal_form_view_name: 'delete_modal_form_view'

    """

    create_form_view_name: str
    create_modal_form_view_name: str
    delete_form_view_name: str
    delete_modal_form_view_name: str
    detail_page_view_name: str
    list_page_view_name: str
    update_form_view_name: str
    update_modal_form_view_name: str

    @classmethod
    def build(cls, components: list[str], **_kwargs) -> ViewFunctions:
        return cls(
            create_form_view_name='create_form_view',
            create_modal_form_view_name='create_modal_form_view',
            delete_form_view_name='delete_form_view',
            delete_modal_form_view_name='delete_modal_form_view',
            detail_page_view_name='detail_page_view',
            list_page_view_name='list_page_view',
            update_form_view_name='update_form_view',
            update_modal_form_view_name='update_modal_form_view',
        )


def generate_replacement_map(
    components: list[str],
    user_inputs: dict[str, str] | None = None
) -> dict[str, str]:
    """Generate replacement mappings for template processing."""

    builder = ReplacementMapBuilder.build(components, user_inputs)
    replacement_dict = builder.to_dict()

    return {key: str(value) for key, value in replacement_dict.items()}



from __future__ import annotations

from dataclasses import dataclass
from typing_extensions import ClassVar

from django_spire.theme.enums import ThemeFamily, ThemeMode


@dataclass(frozen=True)
class Theme:
    family: ThemeFamily
    mode: ThemeMode

    DEFAULT_FAMILY: ClassVar[ThemeFamily] = ThemeFamily.DEFAULT
    DEFAULT_MODE: ClassVar[ThemeMode] = ThemeMode.LIGHT
    SEPARATOR: ClassVar[str] = '-'

    FAMILY_DISPLAY_NAMES: ClassVar[dict[ThemeFamily, str]] = {
        ThemeFamily.AYU: 'Ayu',
        ThemeFamily.CATPPUCCIN: 'Catppuccin',
        ThemeFamily.DEFAULT: 'Default',
        ThemeFamily.GRUVBOX: 'Gruvbox',
        ThemeFamily.MATERIAL: 'Material',
        ThemeFamily.NORD: 'Nord',
        ThemeFamily.ONE_DARK: 'One Dark Pro',
        ThemeFamily.PALENIGHT: 'Palenight',
        ThemeFamily.ROSE_PINE: 'Rose Pine',
        ThemeFamily.TOKYO_NIGHT: 'Tokyo Night',
    }

    def __post_init__(self):
        if isinstance(self.family, str):
            self._validate_family(self.family)

            object.__setattr__(
                self,
                'family',
                ThemeFamily(self.family)
            )

        if isinstance(self.mode, str):
            self._validate_mode(self.mode)

            object.__setattr__(
                self,
                'mode',
                ThemeMode(self.mode)
            )

    @classmethod
    def _parse(cls, theme: str) -> tuple[str, str]:
        parts = theme.strip().split(cls.SEPARATOR)

        if len(parts) < 2:
            message = f'Invalid theme format: {theme}'
            raise ValueError(message)

        mode = parts[-1]
        family = cls.SEPARATOR.join(parts[:-1])

        return family, mode

    @classmethod
    def _validate_family(cls, family: str) -> None:
        if family not in [family.value for family in ThemeFamily]:
            message = f'Invalid theme family: {family}'
            raise ValueError(message)

    @classmethod
    def _validate_mode(cls, mode: str) -> None:
        if mode not in [mode.value for mode in ThemeMode]:
            message = f'Invalid theme mode: {mode}'
            raise ValueError(message)

    @classmethod
    def _validate(cls, family: str, mode: str) -> None:
        cls._validate_family(family)
        cls._validate_mode(mode)

    @classmethod
    def from_string(cls, theme: str, default: Theme | None = None) -> Theme:
        if not theme:
            if default:
                return default

            return cls.get_default()

        try:
            family, mode = cls._parse(theme)
            cls._validate(family, mode)
            return cls(family=ThemeFamily(family), mode=ThemeMode(mode))
        except ValueError:
            if default:
                return default

            raise

    @classmethod
    def get_available(cls) -> list[Theme]:
        return [
            cls(family=family, mode=mode)
            for family in ThemeFamily
            for mode in ThemeMode
        ]

    @classmethod
    def get_default(cls) -> Theme:
        return cls(family=cls.DEFAULT_FAMILY, mode=cls.DEFAULT_MODE)

    @property
    def display(self) -> str:
        family = self.FAMILY_DISPLAY_NAMES.get(self.family, self.family.value)
        mode = self.mode.value.capitalize()
        return f'{family} - {mode}'

    @property
    def family_display(self) -> str:
        return self.FAMILY_DISPLAY_NAMES.get(self.family, self.family.value)

    @property
    def is_dark(self) -> bool:
        return self.mode == ThemeMode.DARK

    @property
    def stylesheet(self) -> str:
        return f'django_spire/css/themes/{self.family.value}/app-{self.mode.value}.css'

    @property
    def value(self) -> str:
        return f'{self.family.value}{self.SEPARATOR}{self.mode.value}'

    def to_dict(self) -> dict:
        return {
            'display': self.display,
            'family': self.family.value,
            'family_display': self.family_display,
            'full': self.value,
            'is_dark': self.is_dark,
            'mode': self.mode.value,
            'stylesheet': self.stylesheet,
        }



from django.contrib import admin, messages
from django.db.models import QuerySet
from django.urls import reverse
from django.utils.html import format_html
from django.utils.http import urlencode

from django_spire.knowledge.entry.models import Entry


@admin.register(Entry)
class EntryAdmin(admin.ModelAdmin):
    list_display = ['name', 'current_version_link', 'collection', 'is_deleted', 'tag_count']
    list_select_related = ['collection', 'current_version']
    list_filter = ['is_deleted', 'is_active']
    search_fields = ['name', 'collection__name']
    ordering = ['name']
    autocomplete_fields = ['collection', 'current_version']
    actions = ['set_tags_for_entries']

    def current_version_link(self, entry: Entry) -> str:
        url = (
            reverse('admin:django_spire_knowledge_entryversion_changelist')
            + '?'
            + urlencode({'entry_id': f'{entry.id}'})
        )

        return format_html(f'<a href="{url}">View Versions</a>')

    current_version_link.short_description = 'Current Version'
    current_version_link.allow_tags = True

    @admin.action(description="Set Tags for Entries (Allow 5 Seconds Per)")
    def set_tags_for_entries(self, request, queryset: QuerySet[Entry]):
        processed = 0
        for entry in queryset:
            entry.services.tag.process_and_set_tags()
            processed += 1

        messages.success(request, f'Successfully processed {processed} entries.')

    def tag_count(self, entry: Entry):
        return entry.tags.count()

    tag_count.short_description = 'Tags'



from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.tests.factories import create_test_helpdesk_ticket


class HelpDeskPageViewsTestCase(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.test_ticket = create_test_helpdesk_ticket()

    def test_ticket_delete_view(self) -> None:
        response = self.client.post(
            reverse(
                'django_spire:help_desk:page:delete',
                kwargs={'pk': self.test_ticket.pk}
            ),
            data={'should_delete': 'on'},
        )

        self.assertEqual(response.status_code, 302)
        self.test_ticket.refresh_from_db()
        self.assertTrue(self.test_ticket.is_deleted)

    def test_ticket_detail_view(self) -> None:
        response = self.client.get(
            reverse(
                'django_spire:help_desk:page:detail',
                kwargs={'pk': self.test_ticket.pk}
            ),
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context_data['ticket'], self.test_ticket)

    def test_ticket_list_view(self) -> None:
        tickets = [create_test_helpdesk_ticket() for _ in range(5)]

        response = self.client.get(
            reverse('django_spire:help_desk:page:list'),
        )

        self.assertEqual(response.status_code, 200)
        for ticket in tickets:
            self.assertIn(ticket, response.context_data['tickets'])



from __future__ import annotations

from dandy import BaseIntel, BaseListIntel

from django_spire.knowledge.intelligence.intel.collection_intel import CollectionIntel


class EntryIntel(BaseIntel):
    relevant_heading_text: str
    relevant_subject_text: str
    relevant_block_id: int
    entry_id: int = None
    collection_intel: CollectionIntel = None


class EntriesIntel(BaseListIntel[EntryIntel]):
    entry_intel_list: list[EntryIntel]



from __future__ import annotations

from dandy import Decoder

from django_spire.knowledge.collection.models import Collection


def get_collection_decoder() -> Decoder:
    class CollectionDecoder(Decoder):
        mapping_keys_description = 'Knowledge Collections Tags'
        mapping = {
            **{
                f'{collection.services.tag.get_aggregated_tag_set_simplified()}': collection
                for collection in Collection.objects.all().annotate_entry_count()
            },
            'No Matching Knowledge Collection Tags': None,
        }

    return CollectionDecoder()



from __future__ import annotations

from django.core.exceptions import ValidationError
from django.db import models

from django_spire.ai.context import querysets
from django_spire.ai.context.choices import PersonRoleChoices
from django_spire.auth.user.models import AuthUser
from django_spire.history.mixins import HistoryModelMixin


class Organization(HistoryModelMixin):
    name = models.CharField(max_length=255, blank=True, null=True)
    legal_name = models.CharField(max_length=255, blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    sector = models.CharField(max_length=255, blank=True, null=True)
    sub_sector = models.CharField(max_length=255, blank=True, null=True)
    website = models.URLField(max_length=255, blank=True, null=True)
    street_address = models.CharField(max_length=255, blank=True, null=True)
    unit_number = models.CharField(max_length=32, blank=True, null=True)
    city = models.CharField(max_length=255, blank=True, null=True)
    province = models.CharField(max_length=255, blank=True, null=True)
    postal_code = models.CharField(max_length=16, blank=True, null=True)
    country = models.CharField(max_length=255, blank=True, null=True)
    phone = models.CharField(max_length=255, blank=True, null=True)
    email = models.EmailField(max_length=255, blank=True, null=True)

    objects = querysets.OrganizationQuerySet.as_manager()

    def save(self, *args, **kwargs) -> None:
        if self.pk is None:
            if self.__class__.objects.exists():
                message = 'Only one AI Organization Context is allowed'
                raise ValidationError(message)

        return super().save(*args, **kwargs)

    class Meta:
        db_table = 'django_spire_ai_context_organization'
        verbose_name = 'Organization Context'
        verbose_name_plural = 'Organization Context'


class Person(HistoryModelMixin):
    is_internal_to_organization = models.BooleanField(default=True)
    user = models.ForeignKey(
        AuthUser,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='ai_context_people',
        related_query_name='ai_context_person'
    )
    role = models.CharField(max_length=16, choices=PersonRoleChoices, default=PersonRoleChoices.ADMIN)
    role_details = models.TextField(blank=True, null=True)
    first_name = models.CharField(max_length=255, blank=True, null=True)
    last_name = models.CharField(max_length=255, blank=True, null=True)
    phone = models.CharField(max_length=255, blank=True, null=True)
    email = models.EmailField(max_length=255, blank=True, null=True)

    objects = querysets.PeopleQuerySet.as_manager()

    class Meta:
        db_table = 'django_spire_ai_context_person'
        verbose_name = 'Person Context'
        verbose_name_plural = 'People Context'




from __future__ import annotations

from dandy import Bot, LlmConfigOptions

from django_spire.ai.prompt.system import intel, prompts


class RoleSystemPromptBot(Bot):
    llm_role = prompts.role_bot_prompt()
    llm_config_options = LlmConfigOptions(temperature=0.5)

    def process(self, user_story: str) -> intel.SystemPromptResultIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story),
            intel_class=intel.SystemPromptResultIntel
        )


class TaskSystemPromptBot(Bot):
    llm_role = prompts.task_bot_prompt()
    llm_config_options = LlmConfigOptions(temperature=0.5)

    def process(self, user_story: str) -> intel.SystemPromptResultIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story),
            intel_class=intel.SystemPromptResultIntel
        )


class GuidelinesSystemPromptBot(Bot):
    llm_role = prompts.guidelines_bot_prompt()
    llm_config_options = LlmConfigOptions(temperature=0.2)

    def process(self, user_story: str) -> intel.SystemPromptResultIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story),
            intel_class=intel.SystemPromptResultIntel
        )


class OutputFormatSystemPromptBot(Bot):
    llm_role = prompts.output_format_bot_prompt()
    llm_config_options = LlmConfigOptions(temperature=0.5)

    def process(self, user_story: str) -> intel.SystemPromptResultIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.system_user_input_prompt(user_story),
            intel_class=intel.SystemPromptResultIntel
        )


class SystemPromptBot(Bot):
    llm_role = prompts.system_prompt_instruction_bot_prompt()

    def process(self, user_story: str) -> intel.SystemPromptIntel:
        role_bot = RoleSystemPromptBot()
        task_bot = TaskSystemPromptBot()
        guidelines_bot = GuidelinesSystemPromptBot()
        output_format_bot = OutputFormatSystemPromptBot()

        # role_future = role_bot.process_to_future(user_story)
        # task_future = task_bot.process_to_future(user_story)
        # guidelines_future = guidelines_bot.process_to_future(user_story)
        # output_format_future = output_format_bot.process_to_future(user_story)

        role = role_bot.process(user_story)
        task = task_bot.process(user_story)
        guidelines = guidelines_bot.process(user_story)
        output_format = output_format_bot.process(user_story)

        return intel.SystemPromptIntel(
            role=role.result,
            task=task.result,
            guidelines=guidelines.result,
            output_format=output_format.result
        )



from __future__ import annotations

from django.core.handlers.wsgi import WSGIRequest
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib import Breadcrumbs
from django_spire.contrib.generic_views import portal_views
from django_spire.knowledge.collection.models import Collection


@AppAuthController('knowledge').permission_required('can_view')
def top_level_collection_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    collection = get_object_or_404(Collection, pk=pk)

    breadcrumbs = Breadcrumbs()
    breadcrumbs.add_breadcrumb('Knowledge', reverse('django_spire:knowledge:page:home'))
    breadcrumbs.add_base_breadcrumb(collection)

    return portal_views.template_view(
        request,
        page_title='Knowledge Collection',
        page_description='',
        breadcrumbs=breadcrumbs,
        context_data={
            'collection': collection,
            'collection_tree_json': Collection.services.transformation.to_hierarchy_json(
                request=request, parent_id=collection.id
            ),
        },
        template='django_spire/knowledge/collection/page/display_page.html',
    )


@AppAuthController('knowledge').permission_required('can_delete')
def delete_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    collection = get_object_or_404(Collection, pk=pk)

    if collection.parent:
        return_url = request.GET.get(
            'return_url',
            reverse(
                'django_spire:knowledge:collection:page:top_level',
                kwargs={'pk': collection.parent_id}
            )
        )
    else:
        return_url = request.GET.get('return_url', reverse('django_spire:knowledge:page:home'))

    return portal_views.delete_form_view(
        request,
        obj=collection,
        delete_func=collection.services.processor.set_deleted,
        return_url=return_url
    )



from __future__ import annotations

from django.core.handlers.wsgi import WSGIRequest
from django.template.response import TemplateResponse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib.generic_views import portal_views
from django_spire.knowledge.collection.models import Collection


@AppAuthController('knowledge').permission_required('can_view')
def home_view(request: WSGIRequest) -> TemplateResponse:
    def breadcrumbs_func(breadcrumbs):
        breadcrumbs.add_breadcrumb(name='Knowledge')

    return portal_views.list_view(
        request,
        model=Collection,
        breadcrumbs_func=breadcrumbs_func,
        context_data={
            'collections': Collection.objects.active().parentless().request_user_has_access(request),
        },
        template='django_spire/knowledge/page/home_page.html',
    )



from typing import Any

from django.contrib.contenttypes.models import ContentType

from django_spire.file.models import File


def copy_files_from_source_to_target_model_object(source: Any, target: Any) -> list[File]:
    target_class = target.__class__
    target_content_type = ContentType.objects.get_for_model(target_class)

    file_list = source.files.active()

    if not file_list.exists():
        return []

    files_to_create = []
    for file in file_list:
        new_file = File(
            content_type=target_content_type,
            object_id=target.pk,
            file=file.file,
            name=file.name,
            size=file.size,
            type=file.type,
            related_field=file.related_field,
            is_active=file.is_active,
            is_deleted=file.is_deleted
        )
        files_to_create.append(new_file)

    File.objects.bulk_create(files_to_create)

    return files_to_create



from django.contrib.contenttypes.models import ContentType
from django.db.models import Model

from dandy import Prompt


def generic_relationship_selection_prompt(
    model_class: type[Model],
    related_model_classes: list[type[Model]]
) -> Prompt:
    related_model_classes_map = {
        related_model_class._meta.verbose_name.title(): {
            'content_type_id': ContentType.objects.get_for_model(related_model_class).id,
            'queryset': related_model_class.objects.all(),
        }
        for related_model_class in related_model_classes
    }

    model_name = model_class._meta.verbose_name.title()
    related_model_class_names = list(related_model_classes_map.keys())

    related_model_object_id_to_content_type_id_map = {}

    for related_model_class_map in related_model_classes_map.values():
        for related_model_object in related_model_class_map['queryset']:
            model_class_name = related_model_object._meta.verbose_name.title()
            content_type_id = related_model_classes_map[model_class_name]['content_type_id']
            related_model_object_id_to_content_type_id_map[str(related_model_object.id)] = content_type_id

    return (
        Prompt()
        .heading('Generic Related Model Selection')
        .text(f'Each `{model_name}` must be linked to an existing entity.')
        .text('The following entity types are available to use:')
        .list(related_model_class_names)
        .line_break()
        .text('The following entities are available to assign (IDs listed):')
        .unordered_random_list(list(related_model_object_id_to_content_type_id_map.keys()))
        .line_break()
        .text('Use the following mapping to correctly set `content_type_id` based on the selected entity ID:')
        .dict(related_model_object_id_to_content_type_id_map)
        .line_break()
        .text(f'The `object_id` is the ID of the entity this `{model_name}` is associated with.')
    )



from __future__ import annotations

from typing import Callable, TYPE_CHECKING

from dandy import Decoder

from django_spire.auth.controller.controller import AppAuthController
from django_spire.conf import settings
from django_spire.core.utils import get_callable_from_module_string_and_validate_arguments
from django_spire.knowledge.intelligence.workflows.knowledge_workflow import knowledge_search_workflow

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def generate_intent_decoder(
    request: WSGIRequest,
    default_callable: Callable | None = None
) -> Decoder:
    intent_dict = {}

    if AppAuthController(app_name='knowledge', request=request).can_view():
        intent_dict['The user is looking for information or knowledge on something.'] = knowledge_search_workflow

    if settings.AI_CHAT_DEFAULT_CALLABLE is not None:
        intent_dict['None of the above choices match the user\'s intent'] = get_callable_from_module_string_and_validate_arguments(
            settings.AI_CHAT_DEFAULT_CALLABLE,
            ['request', 'user_input', 'message_history']
        )

    return Decoder(
        mapping_keys_description='Intent of the User\'s Request',
        mapping=intent_dict
    )



from __future__ import annotations

from string import Template
from typing import TYPE_CHECKING

from django_spire.core.management.commands.spire_startapp_pkg.maps import generate_replacement_map

if TYPE_CHECKING:
    from pathlib import Path

    from django_spire.core.management.commands.spire_startapp_pkg.filesystem import FileSystem


class TemplateEngine:
    """
    Renders template strings with variable replacements.

    This class uses Python's string.Template to safely substitute
    placeholders in template files with actual values.
    """

    def render(self, text: str, replacements: dict[str, str]) -> str:
        """
        Renders a template string by replacing placeholders with values.

        :param text: Template string containing ${variable} placeholders.
        :param replacements: Dictionary mapping placeholder names to their values.
        :return: Rendered string with all placeholders replaced.
        """

        template = Template(text)
        return template.safe_substitute(replacements)


class TemplateProcessor:
    """
    Processes template files for Django app generation.

    This class handles the replacement of placeholders in template files
    and manages file renaming based on user configuration.
    """

    def __init__(self, engine: TemplateEngine, filesystem: FileSystem):
        """
        Initializes the processor with an engine and file system.

        :param engine: Template engine for rendering strings.
        :param filesystem: File system for file operations.
        """

        self._engine = engine
        self._filesystem = filesystem

    def process_app_templates(
        self,
        directory: Path,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> None:
        """
        Processes all template files in an app directory.

        Replaces placeholders in .template files, renames them, and updates
        content in .py files based on user inputs.

        :param directory: Root directory containing template files.
        :param components: List of app path components.
        :param user_inputs: Optional dictionary of user-provided configuration values.
        """

        for file_path in self._filesystem.iterate_files(directory, '*.template'):
            self._process_file(file_path, components, user_inputs)

        for file_path in self._filesystem.iterate_files(directory, '*.py'):
            self._replace_content(file_path, components, user_inputs)
            self._rename_file(file_path, components, user_inputs)

        self._rename_template_files(directory)

    def process_html_templates(
        self,
        directory: Path,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> None:
        """
        Processes all HTML template files in a directory.

        Replaces placeholders in .template files and renames them to remove
        the .template extension.

        :param directory: Root directory containing HTML template files.
        :param components: List of app path components.
        :param user_inputs: Optional dictionary of user-provided configuration values.
        """

        for file_path in self._filesystem.iterate_files(directory, '*.template'):
            self._process_file(file_path, components, user_inputs)

        self._rename_template_files(directory)

    def _process_file(
        self,
        path: Path,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> None:
        """
        Processes a single template file.

        Replaces content placeholders and renames the file based on configuration.

        :param path: Path to the template file.
        :param components: List of app path components.
        :param user_inputs: Optional dictionary of user-provided configuration values.
        """

        self._replace_content(path, components, user_inputs)
        self._rename_file(path, components, user_inputs)

    def _rename_file(
        self,
        path: Path,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> None:
        """
        Renames a file by replacing placeholders in its filename.

        :param path: Current path of the file.
        :param components: List of app path components.
        :param user_inputs: Optional dictionary of user-provided configuration values.
        """

        replacement = generate_replacement_map(components, user_inputs)
        new_name = self._engine.render(path.name, replacement)

        if new_name != path.name:
            new_path = path.parent / new_name
            self._filesystem.rename(path, new_path)

    def _rename_template_files(self, directory: Path) -> None:
        """
        Removes .template extensions from all template files in a directory.

        :param directory: Directory to search for .template files.
        """

        for template_file in self._filesystem.iterate_files(directory, '*.template'):
            new_name = template_file.name.replace('.template', '')
            new_path = template_file.parent / new_name
            self._filesystem.rename(template_file, new_path)

    def _replace_content(
        self,
        path: Path,
        components: list[str],
        user_inputs: dict[str, str] | None = None
    ) -> None:
        """
        Replaces placeholders in a file's content.

        :param path: Path to the file to process.
        :param components: List of app path components.
        :param user_inputs: Optional dictionary of user-provided configuration values.
        """

        replacement = generate_replacement_map(components, user_inputs)
        content = self._filesystem.read_file(path)
        updated_content = self._engine.render(content, replacement)
        self._filesystem.write_file(path, updated_content)



from enum import Enum
from unittest import TestCase

from django_spire.core.converters.to_enums import django_choices_to_enums


class TestDjangoChoicesToEnums(TestCase):

    def test_basic_conversion(self):
        """Test basic conversion with simple choices"""
        choices = [
            ('ACTIVE', 'Active'),
            ('INACTIVE', 'Inactive'),
            ('PENDING', 'Pending')
        ]

        ResultEnum = django_choices_to_enums('Status', choices)

        # Check if enum is created correctly
        self.assertTrue(issubclass(ResultEnum, Enum))
        self.assertEqual(ResultEnum.__name__, 'Status')

        # Check enum members
        self.assertEqual(ResultEnum.ACTIVE.value, 'ACTIVE')
        self.assertEqual(ResultEnum.INACTIVE.value, 'INACTIVE')
        self.assertEqual(ResultEnum.PENDING.value, 'PENDING')

        # Check number of members
        self.assertEqual(len(ResultEnum), 3)

    def test_empty_choices(self):
        """Test function with empty choices list"""
        choices = []
        ResultEnum = django_choices_to_enums('EmptyStatus', choices)

        self.assertTrue(issubclass(ResultEnum, Enum))
        self.assertEqual(ResultEnum.__name__, 'EmptyStatus')
        self.assertEqual(len(ResultEnum), 0)

    def test_single_choice(self):
        """Test with a single choice"""
        choices = [('OPEN', 'Open')]
        ResultEnum = django_choices_to_enums('SingleStatus', choices)

        self.assertTrue(issubclass(ResultEnum, Enum))
        self.assertEqual(ResultEnum.__name__, 'SingleStatus')
        self.assertEqual(ResultEnum.OPEN.value, 'OPEN')
        self.assertEqual(len(ResultEnum), 1)

    def test_duplicate_keys(self):
        """Test behavior with duplicate keys (should use last occurrence)"""
        choices = [
            ('STATUS', 'First'),
            ('STATUS', 'Second')
        ]
        ResultEnum = django_choices_to_enums('DuplicateStatus', choices)

        self.assertTrue(issubclass(ResultEnum, Enum))
        self.assertEqual(ResultEnum.STATUS.value, 'STATUS')
        self.assertEqual(len(ResultEnum), 1)

    def test_different_key_value(self):
        """Test when keys and values differ"""
        choices = [
            ('A', 'Alpha'),
            ('B', 'Beta'),
            ('G', 'Gamma')
        ]
        ResultEnum = django_choices_to_enums('Greek', choices)

        self.assertTrue(issubclass(ResultEnum, Enum))
        self.assertEqual(ResultEnum.A.value, 'A')
        self.assertEqual(ResultEnum.B.value, 'B')
        self.assertEqual(ResultEnum.G.value, 'G')
        self.assertEqual(len(ResultEnum), 3)



from __future__ import annotations

from django.db import models

from django_spire.contrib.ordering.mixins import OrderingModelMixin
from django_spire.history.mixins import HistoryModelMixin
from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices
from django_spire.knowledge.entry.version.block.data.data import BaseEditorJsBlockData
from django_spire.knowledge.entry.version.block.data.maps import EDITOR_JS_BLOCK_DATA_MAP
from django_spire.knowledge.entry.version.block.querysets import \
    EntryVersionBlockQuerySet
from django_spire.knowledge.entry.version.block.services.service import \
    EntryVersionBlockService
from django_spire.knowledge.entry.version.models import EntryVersion


class EntryVersionBlock(HistoryModelMixin, OrderingModelMixin):
    version = models.ForeignKey(
        EntryVersion,
        on_delete=models.CASCADE,
        related_name='blocks',
        related_query_name='block'
    )

    type = models.CharField(
        max_length=32,
        choices=BlockTypeChoices,
        default=BlockTypeChoices.TEXT
    )

    _block_data = models.JSONField()
    _text_data = models.TextField()

    # contains data related to EditorJS tunes,
    # which are additional modifications to blocks (e.g. footnotes, etc.)
    # https://editorjs.io/block-tunes-api/
    _tunes_data = models.JSONField(null=True, blank=True)

    objects = EntryVersionBlockQuerySet.as_manager()
    services = EntryVersionBlockService()

    @property
    def editor_js_block_data(self) -> BaseEditorJsBlockData:
        return EDITOR_JS_BLOCK_DATA_MAP[self.type](**self._block_data)

    @editor_js_block_data.setter
    def editor_js_block_data(self, value: BaseEditorJsBlockData):
        # exclude_none=True ensures that block meta objects aren't autofilled with keys,
        # which can mess up editor rendering
        self._block_data = value.model_dump(exclude_none=True)
        self._text_data = value.render_to_text()

    def update_editor_js_block_data_from_dict(self, value: dict):
        self.editor_js_block_data = EDITOR_JS_BLOCK_DATA_MAP[self.type](**value)

    def render_to_text(self) -> str:
        return self.editor_js_block_data.render_to_text()

    class Meta:
        verbose_name = 'Block'
        verbose_name_plural = 'Blocks'
        db_table = 'django_spire_knowledge_entry_version_block'



DJANGO_SPIRE_AUTH_CONTROLLERS = {
    'ai_chat': 'django_spire.ai.chat.auth.controller.BaseAiChatAuthController',
    'help_desk': 'django_spire.help_desk.auth.controller.BaseHelpDeskAuthController',
    'knowledge': 'django_spire.knowledge.auth.controller.BaseKnowledgeAuthController',
}

# AI Settings
AI_PERSONA_NAME = 'AI Assistant'
AI_CHAT_CALLABLE = None
AI_CHAT_DEFAULT_CALLABLE = None
AI_SMS_CONVERSATION_DEFAULT_CALLABLE = None

ORGANIZATION_NAME = 'No Organization Provided'
ORGANIZATION_DESCRIPTION = 'No Description Provided'

# Theme Settings
DJANGO_SPIRE_DEFAULT_THEME = 'default-light'
DJANGO_SPIRE_THEME_PATH = '/static/django_spire/css/themes/{family}/app-{mode}.css'



from __future__ import annotations

from dandy.llm.request.message import MessageHistory, RoleLiteralStr
from django.contrib.auth.models import User
from django.db import models
from django.utils.timezone import now

from django_spire.ai.chat.message_intel import BaseMessageIntel, DefaultMessageIntel
from django_spire.ai.chat.responses import MessageResponse
from django_spire.ai.chat.choices import MessageResponseType
from django_spire.ai.chat.querysets import ChatQuerySet, ChatMessageQuerySet
from django_spire.history.mixins import HistoryModelMixin
from django_spire.utils import get_class_from_string, get_class_name_from_class


class Chat(HistoryModelMixin):
    user = models.ForeignKey(
        User,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='chats',
        related_query_name='chat'
    )

    name = models.CharField(max_length=128)
    last_message_datetime = models.DateTimeField(default=now, editable=False)
    has_unread_messages = models.BooleanField(default=False)

    objects = ChatQuerySet.as_manager()

    def __str__(self):
        if len(self.name) < 48:
            return self.name

        return self.name[:48] + '...'

    def add_message_response(self, message_response: MessageResponse) -> None:
        self.messages.create(
            response_type=message_response.type.value,
            sender=message_response.sender,
            _intel_data=message_response.message_intel.model_dump(),
            _intel_class_name=get_class_name_from_class(message_response.message_intel.__class__),
            is_processed=True,
            is_viewed=True
        )
        self.last_message_datetime = now()
        self.save()

    def generate_message_history(
            self,
            message_count: int = 20,
            exclude_last_message: bool = True
    ) -> MessageHistory:
        message_history = MessageHistory()

        messages = self.messages.newest_by_count(message_count)

        if exclude_last_message:
            messages = messages[1:]

        messages = list(reversed(messages))

        for message in messages:
            message_history.add_message(
                role=message.role,
                content=message.intel.content_to_str()
            )

        return message_history

    @property
    def is_empty(self) -> bool:
        return self.messages.count() == 0

    @property
    def name_shortened(self) -> str:
        if len(self.name) > 24:
            return self.name[:24] + '...'

        return self.name

    class Meta:
        db_table = 'django_spire_ai_chat'
        verbose_name = 'Chat'
        verbose_name_plural = 'Chats'
        ordering = ('-last_message_datetime', 'name')


class ChatMessage(HistoryModelMixin):
    chat = models.ForeignKey(
        Chat,
        on_delete=models.CASCADE,
        related_name='messages',
        related_query_name='message'
    )

    response_type = models.CharField(max_length=32, choices=MessageResponseType)
    sender = models.CharField(max_length=128)
    _intel_data = models.JSONField()
    _intel_class_name = models.TextField()
    is_processed = models.BooleanField(default=False)
    is_viewed = models.BooleanField(default=False)

    objects = ChatMessageQuerySet.as_manager()

    def __str__(self):
        content = self.intel.content_to_str()

        if len(content) < 64:
            return content

        return content[:64] + '...'

    @property
    def intel(self):
        try:
            intel_class: type[BaseMessageIntel] = get_class_from_string(self._intel_class_name)
            return intel_class.model_validate(self._intel_data)

        except ImportError:
            intel_class: type[BaseMessageIntel] = DefaultMessageIntel
            return intel_class.model_validate(
                {'text': str(self._intel_data)}
            )

    @intel.setter
    def intel(self, message_intel: BaseMessageIntel):
        self._intel_class_name = get_class_name_from_class(message_intel.__class__)
        self._intel_data = message_intel.model_dump()

    @property
    def role(self) -> RoleLiteralStr:
        if self.response_type == 'request':
            return 'user'

        if self.response_type == 'response':
            return 'assistant'

        return 'system'

    def to_message_response(self) -> MessageResponse:
        return MessageResponse(
            type=MessageResponseType(self.response_type),
            sender=self.sender,
            message_intel=self.intel,
            message_timestamp=self.created_datetime.strftime('%b %d, %Y at %I:%M %p')
        )

    class Meta:
        db_table = 'django_spire_ai_chat_message'
        verbose_name = 'Chat Message'
        verbose_name_plural = 'Chat Messages'



from __future__ import annotations

from django.db.models import Prefetch

from django_spire.history.querysets import HistoryQuerySet


class EntryVersionQuerySet(HistoryQuerySet):
    def prefetch_blocks(self):
        from django_spire.knowledge.entry.version.block.models import EntryVersionBlock

        return self.prefetch_related(
            Prefetch(
                'blocks',
                queryset=EntryVersionBlock.objects.order_by('order')
            )
        )


from django.utils import timezone
from faker import Faker

from django_spire.contrib.seeding.field.base import BaseFieldSeeder
from django_spire.contrib.seeding.field.enums import FieldSeederTypesEnum


class CustomFieldSeeder(BaseFieldSeeder):
    keyword = FieldSeederTypesEnum.CUSTOM

    def in_order(self, values: list, index: int) -> any:
        if not values:
            raise ValueError(
                "Cannot select from empty values list. "
                "Make sure the related model has existing records before seeding foreign keys."
            )
        index = index % len(values)  # Index loops back on itself
        return values[index]

    def date_time_between(self, start_date: str, end_date: str):
        faker = Faker()
        naive_dt = faker.date_time_between(start_date=start_date, end_date=end_date)
        return timezone.make_aware(naive_dt)

    def fk_random(self, model_class, ids: list[int]):
        faker = Faker()
        return faker.random_element(elements=ids)

    def fk_in_order(self, model_class, index: int, ids: list[int]):
        """Takes a queryset, calls it then returns a random id"""
        return self.in_order(values=ids, index=index)

    def seed(self, manager, count) -> list[dict]:
        data = []

        foreign_keys = {}

        for i in range(count):
            row = {}

            for field_name, config in self.seeder_fields.items():
                method_name = config[1] if len(config) > 1 else field_name
                kwargs = config[2] if len(config) > 2 else {}
                method = getattr(self, method_name, None)

                if not callable(method):
                    message = f"Custom method '{method_name}' not found for field '{field_name}'"
                    raise TypeError(message)

                if method_name in ['in_order', 'fk_in_order']:
                    kwargs["index"] = i

                if method_name in ['fk_random', 'fk_in_order']:
                    model_class = kwargs["model_class"]

                    if model_class not in foreign_keys:
                        foreign_keys[model_class] = list(model_class.objects.all().values_list('id', flat=True))

                    kwargs['ids'] = foreign_keys[model_class]

                row[field_name] = method(**kwargs)

            data.append(row)

        return data



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.theme.enums import ThemeFamily
from django_spire.theme.models import Theme


class ThemeIntegrationTests(BaseTestCase):
    def test_all_themes_have_stylesheets(self) -> None:
        for theme in Theme.get_available():
            stylesheet = theme.stylesheet

            self.assertIn(f'{theme.family.value}/', stylesheet)
            self.assertIn(f'app-{theme.mode.value}.css', stylesheet)

    def test_roundtrip_conversion(self) -> None:
        for theme in Theme.get_available():
            string = theme.value
            recovered = Theme.from_string(string)
            self.assertEqual(theme, recovered)

            dictionary = theme.to_dict()
            recovered2 = Theme.from_string(dictionary['full'])
            self.assertEqual(theme, recovered2)

    def test_display_names_complete(self) -> None:
        for family in ThemeFamily:
            self.assertIn(family, Theme.FAMILY_DISPLAY_NAMES)
            display_name = Theme.FAMILY_DISPLAY_NAMES[family]
            self.assertIsInstance(display_name, str)
            self.assertTrue(len(display_name) > 0)

    def test_theme_integration_with_authenticated_user(self) -> None:
        available = Theme.get_available()
        self.assertGreater(len(available), 0)

        for theme in available[:3]:
            self.client.cookies['app-theme'] = theme.value
            response = self.client.get('/')
            self.assertEqual(response.status_code, 200)

    def test_theme_media_integration(self) -> None:
        theme = Theme(family=ThemeFamily.GRUVBOX, mode='dark')
        stylesheet = theme.stylesheet

        self.assertIn('django_spire/css/themes/', stylesheet)
        self.assertIn('gruvbox', stylesheet)
        self.assertIn('app-dark.css', stylesheet)



from __future__ import annotations

from django.contrib import admin

from django_spire.knowledge.entry.version.block.models import EntryVersionBlock


@admin.register(EntryVersionBlock)
class EntryVersionBlockAdmin(admin.ModelAdmin):
    list_display = ['version__entry__name', 'type', '_block_data', 'is_deleted']
    list_select_related = ['version__entry']
    list_filter = ['type', 'is_deleted', 'is_active']
    search_fields = ['version__entry__name']
    ordering = ['-created_datetime']
    autocomplete_fields = ['version']



from __future__ import annotations

import json

from typing import TYPE_CHECKING

from django.conf import settings
from django.http import HttpResponse
from django.utils.timezone import now

from django_spire.ai.chat.intelligence.workflows.chat_workflow import chat_workflow
from django_spire.ai.chat.models import Chat
from django_spire.ai.chat.responses import MessageResponse
from django_spire.ai.chat.choices import MessageResponseType

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def response_message_render_view(request: WSGIRequest) -> HttpResponse:
    body_data = json.loads(request.body)

    chat = Chat.objects.by_user(request.user).get(id=body_data['chat_id'])

    message_intel = chat_workflow(
        request,
        body_data['message_body'],
        message_history=chat.generate_message_history(),
    )

    current_datetime = now()
    formatted_timestamp = current_datetime.strftime('%b %d, %Y at %I:%M %p')

    response_message = MessageResponse(
        type=MessageResponseType.RESPONSE,
        sender=settings.AI_PERSONA_NAME,
        message_intel=message_intel,
        synthesis_speech=body_data.get('synthesis_speech', False),
        message_timestamp=formatted_timestamp
    )

    chat.add_message_response(response_message)

    return HttpResponse(response_message.render_to_html_string({'chat_id': chat.id}))



from __future__ import annotations

from django_spire.ai.chat.message_intel import BaseMessageIntel
from django_spire.knowledge.intelligence.intel.entry_intel import EntriesIntel


class KnowledgeMessageIntel(BaseMessageIntel):
    _template: str = 'django_spire/knowledge/message/knowledge_message_intel.html'
    body: str
    entries_intel: EntriesIntel

    def content_to_str(self) -> str:
        return self.body



from __future__ import annotations

import json

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.tests.factories import create_test_entry
from django_spire.knowledge.entry.version.block.tests.factories import \
    create_test_block_form_data
from django_spire.knowledge.entry.version.tests.factories import \
    create_test_entry_version


class EntryVersionJsonUrlsTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.test_entry = create_test_entry()
        self.test_entry_version = create_test_entry_version(entry=self.test_entry)
        self.test_entry.current_version = self.test_entry_version
        self.test_entry.save()

    def test_save_version_blocks_view(self):
        response = self.client.post(
            reverse(
                'django_spire:knowledge:entry:version:json:update_blocks',
                kwargs={'pk': self.test_entry_version.pk}
            ),
            data=json.dumps([create_test_block_form_data()]),
            content_type='application/json'
        )

        self.assertEqual(response.status_code, 200)



from __future__ import annotations

from typing import TYPE_CHECKING

from django.core.handlers.wsgi import WSGIRequest

from django_spire.ai.chat.message_intel import DefaultMessageIntel, BaseMessageIntel
from django_spire.knowledge.intelligence.bots.entry_search_llm_bot import EntrySearchBot
from django_spire.knowledge.intelligence.intel.entry_intel import EntriesIntel
from django_spire.knowledge.intelligence.intel.message_intel import KnowledgeMessageIntel
from django_spire.knowledge.intelligence.decoders.collection_decoder import get_collection_decoder
from django_spire.knowledge.intelligence.decoders.entry_decoder import get_entry_decoder

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from dandy.llm.request.message import MessageHistory

NO_KNOWLEDGE_MESSAGE_INTEL = DefaultMessageIntel(
    text='Sorry, I could not find any information on that.'
)


def knowledge_search_workflow(
        request: WSGIRequest,
        user_input: str,
        message_history: MessageHistory,
) -> BaseMessageIntel | None:
    collection_decoder = get_collection_decoder()
    collections = collection_decoder.process(user_input).values

    if collections[0] is None:
        return NO_KNOWLEDGE_MESSAGE_INTEL

    entries = []

    for collection in collections:
        if collection.entry_count > 0:
            entry_decoder = get_entry_decoder(collection=collection)

            entries.extend(entry_decoder.process(user_input).values)

    entries = [entry for entry in entries if entry is not None]

    if not entries:
        return NO_KNOWLEDGE_MESSAGE_INTEL

    entries_intel = EntriesIntel(entry_intel_list=[])

    for entry in entries:
        entries_intel.append(
            EntrySearchBot().process(user_input=user_input, entry=entry)
        )

    return KnowledgeMessageIntel(body=f'Entries: {entries_intel}', entries_intel=entries_intel)



from __future__ import annotations

from django_spire.knowledge.entry.version.block.data.data import BaseEditorJsBlockData


class TextEditorBlockData(BaseEditorJsBlockData):
    text: str

    def render_to_text(self) -> str:
        from django_spire.knowledge.entry.version.converters.markdown_converter import \
            MarkdownConverter

        return f'{MarkdownConverter.html_to_markdown(self.text)}\n'



from __future__ import annotations

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.tests.factories import create_test_entry


class EntryJsonUrlsTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.test_entry = create_test_entry()

    def test_reorder_view_url_path(self):
        response = self.client.post(
            reverse('django_spire:knowledge:entry:json:reorder'),
            data={'entry_id': self.test_entry.pk, 'order': 0},
            content_type='application/json',
        )

        self.assertEqual(response.status_code, 200)

    def test_update_files_view_url_path(self):
        response = self.client.post(
            reverse('django_spire:knowledge:entry:json:update_files'),
            content_type='application/json',
        )

        self.assertEqual(response.status_code, 200)



from django.db import models
from django.db.models import QuerySet

from django_spire.core.tags.models import Tag
from django_spire.core.tags import tools


class TagsModelMixin(models.Model):
    tags = models.ManyToManyField(Tag, related_name='+', null=True, blank=True, editable=False)

    class Meta:
        abstract = True

    @property
    def tag_set(self) -> set[str]:
        return set(self.tags.values_list('name', flat=True))

    @property
    def tag_set_simplified(self) -> set[str]:
        simplified_tag_words = []

        for tag in self.tag_set:
            tag_words = tag.split('-')
            simplified_tag_words.extend(tag_words)

        return set(simplified_tag_words)

    def add_tags_from_tag_set(self, tag_set: set[str]):
        self._update_global_tags_from_set(tag_set)

        self.tags.add(*Tag.objects.in_tag_set(tag_set))

    def clear_tags(self):
        self.tags.clear()

    def get_matching_tags_from_tag_set(self, tag_set: set[str]) -> QuerySet:
        return self.tags.in_tag_set(tag_set)

    def get_matching_percentage_of_tag_set(self, tag_set: set[str]) -> float:
        return tools.get_matching_a_percentage_from_tag_sets(tag_set, self.tag_set)

    def get_matching_percentage_of_model_tags_from_tag_set(self, tag_set: set[str]) -> float:
        return tools.get_matching_b_percentage_from_tag_sets(tag_set, self.tag_set)

    def has_tags_in_tag_set(self, tag_set: set[str]) -> bool:
        return self.tag_set.issuperset(tag_set)

    def remove_tags_by_tag_set(self, tag_set: set[str]):
        tag_objects = Tag.objects.in_tag_set(tag_set)

        self.tags.remove(*tag_objects)

    def set_tags_from_tag_set(self, tag_set: set[str]):
        self._update_global_tags_from_set(tag_set)

        tag_objects = Tag.objects.in_tag_set(tag_set)
        self.tags.set(tag_objects)

    @staticmethod
    def _update_global_tags_from_set(tag_set: set[str]):
        Tag.add_tags([Tag(name=tag) for tag in tag_set])



from __future__ import annotations

from django.conf import settings
from django.core.mail import EmailMessage

from django_spire.file.models import File
from django_spire.notification.models import Notification


class EmailHelper:
    def __init__(self, notification: Notification, fail_silently: bool = False):
        email = notification.email
        if isinstance(email.to_email_address, str):
            self.to = [email.to_email_address]
        else:
            self.to = email.to_email_address

        if isinstance(email.cc, str):
            self.cc = [email.cc]
        else:
            self.cc = email.cc

        if isinstance(email.bcc, str):
            self.bcc = [email.bcc]
        else:
            self.bcc = email.bcc

        self.from_email = settings.DEFAULT_FROM_EMAIL
        self.fail_silently = fail_silently

        self.attachments = []
        for attachment in list(email.attachments.all()):
            with attachment.file.open('rb') as f:
                self.attachments.append((attachment.name, f.read(), attachment.type))


class SendGridEmailHelper(EmailHelper):
    def __init__(
        self,
        notification: Notification,
        fail_silently: bool = False
    ):
        super().__init__(notification, fail_silently)

        if notification.email.template_id == '':
            self.template_id = settings.SENDGRID_TEMPLATE_ID

        else:
            self.template_id = notification.email.template_id

        self.template_data = {
            'to': notification.email.to_email_address,
            'from': settings.DEFAULT_FROM_EMAIL,
            'subject': notification.title,
            'body': notification.body,
            'url': notification.url
        }

        self.template_data.update(notification.email.context_data)

    def send(self) -> None:
        msg = EmailMessage(
            from_email=self.from_email,
            to=self.to,
            cc=self.cc,
            bcc=self.bcc,
            attachments=self.attachments
        )

        msg.template_id = self.template_id
        msg.dynamic_template_data = self.template_data
        msg.send(fail_silently=self.fail_silently)



from __future__ import annotations

import json

from django.contrib.sites.models import Site
from django.conf import settings
from django.test import RequestFactory

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.collection.models import Collection


class TestCollectionTransformationService(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.site = Site.objects.create(domain='', name='')

    def _test_collection_urls_json(self, collection_json: dict):
        response = self.client.get(collection_json['delete_url'].replace(' ', ''))
        self.assertEqual(response.status_code, 200)

        response = self.client.get(collection_json['create_entry_url'].replace(' ', ''))
        self.assertEqual(response.status_code, 200)

        response = self.client.get(collection_json['import_entry_url'].replace(' ', ''))
        self.assertEqual(response.status_code, 200)

    # Todo: Talk to Chase about this
    # def test_to_hierarchy_json(self):
    #     settings.SITE_ID = self.site.id
    #
    #     self.test_collection_1 = Collection.objects.create(name='Grand Parent A', id=1, parent_id=None)
    #     self.test_collection_2 = Collection.objects.create(name='Parent A1', id=2, parent_id=1)
    #     self.test_collection_3 = Collection.objects.create(name='Child A1a', id=3, parent_id=2)
    #
    #     request = RequestFactory().get('/')
    #     request.user = self.super_user
    #
    #     family_tree = Collection.services.transformation.to_hierarchy_json(
    #         request=request
    #     )
    #
    #     for collection_json in json.loads(family_tree):
    #         self.assertIsNotNone(collection_json['id'])
    #         self.assertEqual(collection_json['name'], 'Grand Parent A')
    #
    #         self.assertEqual(len(collection_json['entries']), 0)
    #
    #         self._test_collection_urls_json(collection_json=collection_json)
    #
    #         self.assertEqual(len(collection_json['children']), 1)
    #
    #         for child_json in collection_json['children']:
    #             self.assertIsNotNone(child_json['id'])
    #             self.assertEqual(child_json['name'], 'Parent A1')
    #
    #             self.assertEqual(len(child_json['entries']), 0)
    #
    #             self._test_collection_urls_json(collection_json=child_json)
    #
    #             self.assertEqual(len(child_json['children']), 1)
    #
    #             for grand_child_json in child_json['children']:
    #                 self.assertIsNotNone(grand_child_json['id'])
    #                 self.assertEqual(grand_child_json['name'], 'Child A1a')
    #
    #                 self.assertEqual(len(grand_child_json['entries']), 0)
    #
    #                 self._test_collection_urls_json(collection_json=grand_child_json)
    #
    #                 self.assertEqual(len(grand_child_json['children']), 0)



from __future__ import annotations

from django_spire.knowledge.collection.seeding.seeder import CollectionSeeder

parent_collections = CollectionSeeder.seed_database(count=5)
child_collections = CollectionSeeder.seed_child_collections(count=15)

from django_spire.knowledge.entry.seeding.seed import *



from django.db.models import QuerySet
from django.utils.text import slugify


class TagQuerySet(QuerySet):
    def in_tag_set(self, tag_set: set[str]):
        slugified_tag_set = set(map(slugify, tag_set))

        return self.filter(name__in=slugified_tag_set)



from __future__ import annotations

from django_spire.knowledge.entry.version.converters.docx_converter import \
    DocxConverter
from django_spire.knowledge.entry.version.converters.markdown_converter import \
    MarkdownConverter

FILE_TYPE_CONVERTER_MAP = {
    'md': MarkdownConverter,
    'docx': DocxConverter
}


from __future__ import annotations

from dandy import Decoder

from django_spire.knowledge.collection.models import Collection


def get_entry_decoder(collection: Collection) -> Decoder:
    class EntryDecoder(Decoder):
        mapping_keys_description = 'Knowledge Entries Tags'
        mapping = {
            **{
                f'{entry.tag_set}': entry
                for entry in collection.entries.all()
              },
            'No Matching Knowledge Entries': None
        }

    return EntryDecoder()



from __future__ import annotations

from typing import TYPE_CHECKING, Callable

from dandy import Bot
from dandy.recorder import recorder_to_html_file

from django_spire.ai.chat.intelligence.decoders.tools import generate_intent_decoder
from django_spire.ai.chat.message_intel import BaseMessageIntel, DefaultMessageIntel
from django_spire.ai.decorators import log_ai_interaction_from_recorder
from django_spire.conf import settings
from django_spire.core.utils import get_callable_from_module_string_and_validate_arguments

if TYPE_CHECKING:
    from dandy.llm.request.message import MessageHistory
    from django.core.handlers.wsgi import WSGIRequest


def default_chat_callable(
    request: WSGIRequest,
    user_input: str,
    message_history: MessageHistory | None = None
) -> BaseMessageIntel:
    bot = Bot()
    return bot.llm.prompt_to_intel(
        prompt=user_input,
        intel_class=DefaultMessageIntel,
        message_history=message_history,
    )


@recorder_to_html_file('spire_ai_chat_workflow')
def chat_workflow(
    request: WSGIRequest,
    user_input: str,
    message_history: MessageHistory | None = None
) -> BaseMessageIntel:
    @log_ai_interaction_from_recorder(request.user)
    def run_workflow_process(callable_: Callable) -> BaseMessageIntel | None:
        return callable_(
            request=request,
            user_input=user_input,
            message_history=message_history,
        )

    if settings.AI_CHAT_CALLABLE is not None:
        chat_callable = get_callable_from_module_string_and_validate_arguments(
            settings.AI_CHAT_CALLABLE,
            ['request', 'user_input', 'message_history']
        )

        message_intel = run_workflow_process(chat_callable)

    else:

        intent_decoder = generate_intent_decoder(
            request=request,
            default_callable=default_chat_callable,
        )

        intent_process = intent_decoder.process(user_input, max_return_values=1)[0]

        message_intel = run_workflow_process(intent_process)

    if not isinstance(message_intel, BaseMessageIntel):
        if message_intel is None:
            return default_chat_callable(
                request=request,
                user_input=user_input,
                message_history=message_history
            )

        message = f'{intent_process.__qualname__} must return an instance of a {BaseMessageIntel.__name__} sub class.'
        raise TypeError(message)

    return message_intel



from __future__ import annotations

from django_spire.core.decorators import close_db_connections
from django_spire.notification.managers import NotificationManager


@close_db_connections
def process_notifications() -> str:
    NotificationManager().process_ready_notifications()

    return 'Successfully Completed'



from __future__ import annotations

import random
import string

from typing_extensions import Sequence, TYPE_CHECKING, TypeVar

from django import template
from django.db.models import Model
from django.urls import reverse

if TYPE_CHECKING:
    from django.template import RequestContext


register = template.Library()

T = TypeVar('T', bound=Model)
U = TypeVar('U')


@register.filter
def add_str(str1: str, str2: str) -> str:
    """
    Concatenates two strings.

    :param str1: The first string.
    :param str2: The second string.
    :return: The concatenated string.
    """

    return f'{str1}{str2}'


@register.simple_tag()
def content_type_url(url_name: str, obj: T, **kwargs) -> str:
    """
    Constructs a URL for a given content type using the object's metadata.

    :param url_name: The name of the URL pattern.
    :param obj: A Django model instance whose metadata is used to construct the URL.
    :param kwargs: Additional keyword arguments for URL reversal.
    :return: The reversed URL as a string.
    """

    kwargs['app_label'] = obj._meta.app_label
    kwargs['model_name'] = obj._meta.model_name
    return reverse(url_name, kwargs=kwargs)


@register.filter
def in_list(value: str, arg: str) -> bool:
    """
    Checks if a string is present in a comma-separated list.

    :param value: The string to check.
    :param arg: A comma-separated string of values.
    :return: True if the value is present, False otherwise.
    """

    return value in arg.split(',')


@register.filter
def index(indexable: Sequence[U], index_value: int) -> U | Sequence[U]:
    """
    Returns the element at the given index from an indexable or
    the entire indexable if the index is out-of-bounds.

    :param indexable: A sequence from which to retrieve an element.
    :param index_value: The index of the element to retrieve.
    :return: The element at the specified index, or the original sequence
    if the index is out-of-bounds.
    """

    try:
        return indexable[index_value]
    except IndexError:
        return indexable


@register.filter(name='is_path')
def is_path(current: str, url: str) -> bool:
    if not current or not url:
        return False

    if current == url:
        return True

    return url != '/' and current.startswith(url)


@register.simple_tag()
def generate_id() -> str:
    """
    Generates an 8-character random string using ASCII letters.

    :return: An 8-character random string.
    """

    random_string = ''

    for _ in range(8):
        random_string = random_string + random.choice(string.ascii_letters)

    return random_string


@register.filter
def not_in_list(value: str, arg: str) -> bool:
    """
    Checks if a string is not present in a comma-separated list.

    :param value: The string to check.
    :param arg: A comma-separated string of values.
    :return: True if the value is not present, False otherwise.
    """

    return value not in arg.split(',')


@register.simple_tag()
def query_param_url(context: RequestContext, url_name: str, **kwargs) -> str:
    """
    Generates a URL by appending the query parameters from the
    request context to a reversed URL.

    :param context: A RequestContext containing the current request.
    :param url_name: The name of the URL pattern.
    :param kwargs: Additional keyword arguments for URL reversal.
    :return: The generated URL with query parameters appended as a string.
    """

    query_string = '?'

    for index, query_param in enumerate(context.request.GET):
        if index == 0:
            query_string = (
                query_string +
                f'{query_param}={context.request.GET[query_param]}'
            )
        else:
            query_string = (
                query_string +
                f'&{query_param}={context.request.GET[query_param]}'
            )

    return reverse(url_name, kwargs=kwargs) + query_string


@register.simple_tag()
def to_snake_case(label: str) -> str:
    """
    Converts a label to snake_case by replacing spaces with underscores
    and then converting it to lowercase.

    :param label: The label string to convert.
    :return: The snake_case version of the label.
    """

    return label.replace(' ', '_').lower()



from django.db.models import IntegerChoices


class OutcomeRatingChoices(IntegerChoices):
    TERRIBLE = 1, 'terrible'
    BAD = 2, 'bad'
    OK = 3, 'ok'
    GOOD = 4, 'good'
    GREAT = 5, 'great'
    AMAZING = 6, 'amazing'



from django_spire.knowledge.entry.version.block.data.heading_data import \
    HeadingEditorBlockData
from django_spire.knowledge.entry.version.block.data.text_data import \
    TextEditorBlockData

LADDER_SAFETY_BLOCKS = [
    HeadingEditorBlockData(text='Ladder Safety: A Journey Through the Heights',
                           level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Great Ladder Adventure Begins', level=2),
    TextEditorBlockData(
        text='Welcome to the thrilling world of ladder safety! Grab your hard hats and let\'s explore the wonderful dangers of working at heights.'),
    TextEditorBlockData(
        text='Whether you\'re a seasoned climber or a first-time ladder user, this guide will help you navigate the exciting landscape of aerial safety.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='High vs. Low: The Ladder Tightrope', level=2),
    TextEditorBlockData(
        text='High ladders are like climbing Mount Everest - exciting but dangerous! Always double-check your footing and support.'),
    TextEditorBlockData(
        text='Low ladders might seem harmless, but they\'re like slippery snakes waiting to trip you up! Keep them out of high-traffic areas.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Ladder Safety Toolkit', level=2),
    TextEditorBlockData(
        text='Inspect your ladder like a detective looking for clues - check for cracks, rust, or missing rungs.'),
    TextEditorBlockData(
        text='Remember: Three points of contact keeps you safe and sound!'),
    TextEditorBlockData(
        text='Never exceed the weight limit - your ladder has a maximum capacity just like you have a maximum capacity for pizza!'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency: The Ladder Rescue Mission', level=2),
    TextEditorBlockData(
        text='If you find yourself in a precarious situation, stay calm and follow the emergency protocols.'),
    TextEditorBlockData(
        text='First aid training is like having a superhero power - it\'s invaluable when you need it most!'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Ladder Safety Certificate', level=2),
    TextEditorBlockData(
        text='Congratulations! You\'ve completed your ladder safety course. Now go out there and climb with confidence!'),
]

KITCHEN_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Kitchen Safety: A Culinary Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Great Kitchen Escape', level=2),
    TextEditorBlockData(
        text='Welcome to the thrilling world of kitchen safety! Every chef needs to know how to navigate the kitchen without getting burned, cut, or startled by hot oil.'),
    TextEditorBlockData(
        text='Whether you\'re a professional chef or a home cook, these safety tips will help you avoid kitchen disasters and keep your cooking adventures safe.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Hot Oil: The Dangerous Dance', level=2),
    TextEditorBlockData(
        text='Hot oil is like a wild animal - it can splash and burn in seconds! Always use proper protective gear when handling hot oil.'),
    TextEditorBlockData(
        text='Never leave hot oil unattended, or you\'ll end up with a kitchen fire that\'s more exciting than your favorite TV show!'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Knife Safety: The Sharp Edge of Responsibility',
                           level=2),
    TextEditorBlockData(
        text='Knives are like any other tool - they\'re only as dangerous as the person using them.'),
    TextEditorBlockData(
        text='Always cut away from your body and keep your knives sharp (dull knives are actually more dangerous than sharp ones).'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Kitchen Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a kitchen emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having a fire extinguisher in the kitchen is like having a superhero sidekick - it\'s always good to have backup!'),
]

GARDEN_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Garden Safety: A Green Thumb Guide', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Garden Adventure Begins', level=2),
    TextEditorBlockData(
        text='Step into the wonderful world of gardening with safety as your companion! Every gardener should know how to protect themselves from thorns, chemicals, and unexpected wildlife.'),
    TextEditorBlockData(
        text='Whether you\'re tending to a small potted plant or a large garden, these tips will keep you safe while you nurture your green thumb.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Chemical Safety: The Poisonous Garden', level=2),
    TextEditorBlockData(
        text='Gardening chemicals can be like hidden traps - they look harmless but can cause serious harm if not handled properly.'),
    TextEditorBlockData(
        text='Always read labels and wear protective gear when working with fertilizers, pesticides, or other garden chemicals.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Tool Safety: The Gardener\'s Arsenal', level=2),
    TextEditorBlockData(
        text='Gardening tools are like weapons - they\'re only as dangerous as the person wielding them.'),
    TextEditorBlockData(
        text='Keep your tools sharp and clean to prevent accidents and ensure maximum efficiency in your garden work.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Garden Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a garden emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having a first aid kit in the garden is like having a superhero sidekick - it\'s always good to have backup!'),
]

OFFICE_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Office Safety: A Professional Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Office Escape', level=2),
    TextEditorBlockData(
        text='Welcome to the exciting world of office safety! Every professional should know how to navigate their workspace without tripping, falling, or getting distracted by a rogue cable.'),
    TextEditorBlockData(
        text='Whether you\'re working in a bustling office or a quiet cubicle, these safety tips will help you avoid workplace disasters and keep your productivity high.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Cable Safety: The Hidden Danger', level=2),
    TextEditorBlockData(
        text='Office cables are like snakes waiting to strike - they can trip you up if you\'re not careful!'),
    TextEditorBlockData(
        text='Always secure cables properly and keep walkways clear of obstacles.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Ergonomic Safety: The Comfort Zone', level=2),
    TextEditorBlockData(
        text='Proper ergonomics in the office are like having a personal assistant - they make everything easier and more comfortable.'),
    TextEditorBlockData(
        text='Adjust your chair, monitor, and keyboard to prevent strain and injury.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Office Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter an office emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having a fire extinguisher in the office is like having a superhero sidekick - it\'s always good to have backup!'),
]

POOL_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Swimming Pool Safety: A Water Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Pool Party Begins', level=2),
    TextEditorBlockData(
        text='Dive into the exciting world of pool safety! Every swimmer should know how to protect themselves from drowning, sunburn, and unexpected water hazards.'),
    TextEditorBlockData(
        text='Whether you\'re swimming in a backyard pool or a public facility, these tips will keep you safe while you enjoy the water.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Water Safety: The Deep End', level=2),
    TextEditorBlockData(
        text='Pool water can be deceiving - it looks calm but can hide dangerous situations!'),
    TextEditorBlockData(
        text='Always swim with a buddy and never dive into shallow water.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Sun Protection: The Beach Adventure', level=2),
    TextEditorBlockData(
        text='Sun exposure is like a wild animal - it can burn you in seconds!'),
    TextEditorBlockData(
        text='Apply sunscreen regularly and wear protective clothing to avoid sunburn and skin damage.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Pool Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a pool emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having lifeguards on duty is like having a superhero sidekick - it\'s always good to have backup!'),
]

CONSTRUCTION_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Construction Safety: A Building Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Construction Zone', level=2),
    TextEditorBlockData(
        text='Step into the exciting world of construction safety! Every builder should know how to protect themselves from falling objects, electrical hazards, and unexpected structural issues.'),
    TextEditorBlockData(
        text='Whether you\'re working on a skyscraper or a small home renovation, these safety tips will keep you safe while you build your dreams.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Falling Object Safety: The Sky is Falling', level=2),
    TextEditorBlockData(
        text='Construction sites are like a battlefield - falling objects can strike at any moment!'),
    TextEditorBlockData(
        text='Always wear proper head protection and stay clear of work zones.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Electrical Safety: The Power Adventure', level=2),
    TextEditorBlockData(
        text='Electricity is like a wild animal - it can shock you in seconds!'),
    TextEditorBlockData(
        text='Always check electrical equipment before use and never work on live circuits.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Construction Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a construction emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having safety officers on site is like having a superhero sidekick - it\'s always good to have backup!'),
]

CYCLING_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Bicycle Safety: A Cycling Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Road Ahead', level=2),
    TextEditorBlockData(
        text='Ride into the exciting world of bicycle safety! Every cyclist should know how to protect themselves from traffic, weather hazards, and mechanical failures.'),
    TextEditorBlockData(
        text='Whether you\'re commuting or going for a weekend ride, these tips will keep you safe while you enjoy the open road.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Traffic Safety: The Highway Adventure', level=2),
    TextEditorBlockData(
        text='Riding in traffic is like navigating a busy city - always stay alert and follow traffic rules!'),
    TextEditorBlockData(
        text='Always wear a helmet and make sure your bike is properly equipped with lights and reflectors.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Weather Safety: The Elements Adventure', level=2),
    TextEditorBlockData(
        text='Weather conditions can change quickly - always check the forecast before heading out!'),
    TextEditorBlockData(
        text='Avoid riding in rain or snow without proper gear and equipment.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Bicycle Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a bicycle emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having a repair kit on hand is like having a superhero sidekick - it\'s always good to have backup!'),
]

FIRE_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Fire Safety: A Blaze Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Fire Escape', level=2),
    TextEditorBlockData(
        text='Enter the exciting world of fire safety! Every person should know how to protect themselves from flames, smoke, and unexpected fires.'),
    TextEditorBlockData(
        text='Whether you\'re at home or in a public building, these tips will keep you safe while you navigate fire hazards.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Fire Prevention: The Prevention Adventure', level=2),
    TextEditorBlockData(
        text='Prevention is better than cure - always keep fire extinguishers accessible and never leave candles unattended!'),
    TextEditorBlockData(
        text='Keep flammable materials away from heat sources and maintain proper ventilation.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Evacuation Safety: The Escape Route', level=2),
    TextEditorBlockData(
        text='Knowing your escape routes is like having a superhero power - it\'s invaluable when you need it most!'),
    TextEditorBlockData(
        text='Always practice evacuation drills and keep emergency contact information handy.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Fire Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a fire emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having fire safety training is like having a superhero sidekick - it\'s always good to have backup!'),
]

HIKING_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Hiking Safety: A Mountain Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Trail Ahead', level=2),
    TextEditorBlockData(
        text='Embark on the exciting world of hiking safety! Every hiker should know how to protect themselves from weather hazards, wildlife encounters, and unexpected terrain.'),
    TextEditorBlockData(
        text='Whether you\'re taking a short walk or a multi-day trek, these tips will keep you safe while you explore nature.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Weather Safety: The Elements Adventure', level=2),
    TextEditorBlockData(
        text='Weather conditions can change quickly in the wilderness - always check forecasts before heading out!'),
    TextEditorBlockData(
        text='Pack appropriate gear for all weather conditions and never hike alone in dangerous areas.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Wildlife Safety: The Animal Encounter', level=2),
    TextEditorBlockData(
        text='Wild animals are like wild cards - they can appear at any moment and behave unpredictably!'),
    TextEditorBlockData(
        text='Keep food stored properly and maintain a safe distance from wildlife.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Hiking Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a hiking emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having a first aid kit and emergency communication device is like having a superhero sidekick - it\'s always good to have backup!'),
]

BOATING_SAFETY_BLOCKS = [
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Boating Safety: A Water Adventure', level=1),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='The Ocean Escape', level=2),
    TextEditorBlockData(
        text='Set sail into the exciting world of boating safety! Every sailor should know how to protect themselves from water hazards, weather conditions, and unexpected emergencies.'),
    TextEditorBlockData(
        text='Whether you\'re on a small boat or a large vessel, these tips will keep you safe while you enjoy the water.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Water Safety: The Ocean Adventure', level=2),
    TextEditorBlockData(
        text='Boating on water is like navigating a wild ocean - always stay alert and follow safety protocols!'),
    TextEditorBlockData(
        text='Always wear life jackets and check weather conditions before heading out.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Navigation Safety: The Compass Adventure', level=2),
    TextEditorBlockData(
        text='Proper navigation is like having a superhero power - it\'s invaluable when you need it most!'),
    TextEditorBlockData(
        text='Use proper navigation equipment and never navigate without a backup plan.'),
    TextEditorBlockData(text=''),
    HeadingEditorBlockData(text='Emergency Boating Procedures', level=2),
    TextEditorBlockData(
        text='If you encounter a boating emergency, stay calm and follow established protocols.'),
    TextEditorBlockData(
        text='Having safety equipment on board is like having a superhero sidekick - it\'s always good to have backup!'),
]


SAFETY_BLOCKS = [
    LADDER_SAFETY_BLOCKS,
    KITCHEN_SAFETY_BLOCKS,
    GARDEN_SAFETY_BLOCKS,
    OFFICE_SAFETY_BLOCKS,
    POOL_SAFETY_BLOCKS,
    CONSTRUCTION_SAFETY_BLOCKS,
    CYCLING_SAFETY_BLOCKS,
    FIRE_SAFETY_BLOCKS,
    HIKING_SAFETY_BLOCKS,
    BOATING_SAFETY_BLOCKS,
]



from __future__ import annotations

from typing import TYPE_CHECKING

from django.conf import settings
from django.contrib.sites.models import Site
from django.urls import reverse

from django_spire.contrib.service import BaseDjangoModelService

if TYPE_CHECKING:
    from django.db.models import QuerySet
    from django_spire.knowledge.entry.models import Entry


class EntryTransformationService(BaseDjangoModelService['Entry']):
    obj: Entry

    @staticmethod
    def queryset_to_navigation_list(queryset: QuerySet[Entry]) -> list[dict[str, str]]:
        json_list = []
        for entry in queryset:
            json_list.append(entry.services.transformation.to_dict())

        return json_list

    def to_dict(self):
        site = Site.objects.get_current() if not settings.DEBUG else ''
        current_version = self.obj.current_version

        return {
            'entry_id': self.obj.pk,
            'name': self.obj.name,
            'version_id': current_version.id,
            'author': current_version.author.get_full_name(),
            'last_edit_datetime': (
                current_version.last_edit_datetime.strftime('%Y-%m-%d')
                if current_version.last_edit_datetime else ''
            ),
            'publish_datetime': (
                current_version.published_datetime.strftime('%Y-%m-%d')
                if current_version.published_datetime else ''
            ),
            'status': current_version.status,
            'delete_url': f"""
                {site}{
                    reverse(
                        'django_spire:knowledge:entry:page:delete',
                        kwargs={'pk': self.obj.pk},
                    )
                }
            """,
            'edit_url': f"""
                {site}{
                    reverse(
                        'django_spire:knowledge:entry:form:update',
                        kwargs={
                            'pk': self.obj.pk,
                            'collection_pk': self.obj.collection.pk
                        },
                    )
                }
            """,
            'view_url': f"""
                {site}{
                    reverse(
                        'django_spire:knowledge:entry:version:page:editor',
                        kwargs={'pk': current_version.pk},
                    )
                }
            """,
            'edit_version_url': f"""
                {site}{
                    reverse(
                        'django_spire:knowledge:entry:version:page:editor',
                        kwargs={'pk': current_version.pk},
                    )
                }?view_mode=edit
            """
        }



# Generated by Django 5.2.7 on 2025-11-07 23:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_knowledge', '0006_alter_entryversionblock_type'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='collection',
            options={'ordering': ['name'], 'permissions': [('can_access_all_collections', 'Can Access All Collections'), ('can_change_collection_groups', 'Can Change Collection Groups')], 'verbose_name': 'Collection', 'verbose_name_plural': 'Collections'},
        ),
    ]



from abc import ABC, abstractmethod

from django_spire.notification.choices import NotificationStatusChoices
from django_spire.notification.models import Notification


class BaseNotificationProcessor(ABC):
    @abstractmethod
    def process(self, notification: Notification):
        raise NotImplementedError

    @abstractmethod
    def process_list(self, notifications: list[Notification]):
        raise NotImplementedError

    @abstractmethod
    def process_ready(self):
        raise NotImplementedError

    @abstractmethod
    def process_errored(self):
        raise NotImplementedError

    @staticmethod
    def _update_notifications_to_processing(notifications: list[Notification]):
        for notification in notifications:
            notification.status = NotificationStatusChoices.PROCESSING

        Notification.objects.bulk_update(notifications, ['status'])



from __future__ import annotations

import json

from typing import TYPE_CHECKING

from django.conf import settings
from django.http import HttpResponse
from django.utils.timezone import now

from django_spire.ai.chat.choices import MessageResponseType
from django_spire.ai.chat.message_intel import DefaultMessageIntel
from django_spire.ai.chat.models import Chat
from django_spire.ai.chat.responses import MessageResponse, MessageResponseGroup

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


def request_message_render_view(request: WSGIRequest) -> HttpResponse:
    body_data = json.loads(request.body)

    chat_id = body_data['chat_id']

    current_datetime = now()
    formatted_timestamp = current_datetime.strftime('%b %d, %Y at %I:%M %p')

    if chat_id in {0, '0', ''}:
        chat = Chat.objects.create(
            user=request.user,
            name=body_data['message_body'][:64],
            last_message_datetime=current_datetime
        )
    else:
        chat = (
            Chat.objects
            .by_user(request.user)
            .get(id=chat_id)
        )

        if chat.is_empty:
            chat.name = body_data['message_body'][:64]
            chat.save()

    message_response_group = MessageResponseGroup()

    user_message_response = MessageResponse(
        type=MessageResponseType.REQUEST,
        sender='You',
        message_intel=DefaultMessageIntel(
            text=body_data['message_body']
        ),
        message_timestamp=formatted_timestamp
    )

    message_response_group.add_message_response(
        user_message_response
    )

    chat.add_message_response(user_message_response)

    message_response_group.add_message_response(
        MessageResponse(
            type=MessageResponseType.LOADING_RESPONSE,
            sender='Spire',
            message_intel=DefaultMessageIntel(
                text=body_data['message_body']
            ),
            synthesis_speech=body_data.get('synthesis_speech', False),
        )
    )

    return HttpResponse(
        message_response_group.render_to_html_string(
            context_data={
                "chat_id": chat.id,
                "chat_workflow_name": settings.AI_PERSONA_NAME,
            }
        )
    )



from __future__ import annotations

import django_spire

from dataclasses import dataclass
from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from django_spire.core.management.commands.spire_startapp_pkg.resolver import PathResolver


@dataclass(frozen=True)
class PathConfig:
    """
    Configuration for template directory paths.

    This class holds the paths to the app and HTML template directories
    used for generating new Django apps.
    """

    app_template: Path
    html_template: Path

    @classmethod
    def default(cls) -> PathConfig:
        """
        Creates a default PathConfig with standard template locations.

        :return: PathConfig instance with paths to default app and HTML templates.
        """

        base = Path(django_spire.__file__).parent

        return cls(
            app_template=base / 'core/management/commands/spire_startapp_pkg/template/app',
            html_template=base / 'core/management/commands/spire_startapp_pkg/template/templates'
        )


@dataclass(frozen=True)
class AppConfig:
    """
    Configuration for a new Django app being created.

    This class contains all the information needed to generate a new app,
    including its name, path, components, destinations, and user-provided inputs.
    """

    app_name: str
    app_path: str
    components: list[str]
    destination: Path
    template_destination: Path
    user_inputs: dict[str, str]

    @property
    def app_label(self) -> str:
        """
        Gets the Django app label.

        :return: The app label, either from user input or derived from app name.
        """

        return self.user_inputs.get('app_label', self.app_name.lower())

    @property
    def model_name(self) -> str:
        """
        Gets the model class name.

        :return: The model name, either from user input or TitleCase version of app name.
        """

        default = ''.join(word.title() for word in self.app_name.split('_'))
        return self.user_inputs.get('model_name', default)


class AppConfigFactory:
    """
    Factory class for creating AppConfig instances.

    This class handles the creation of AppConfig objects by resolving
    paths and processing user inputs.
    """

    def __init__(self, path_resolver: PathResolver):
        """
        Initializes the factory with a path resolver.

        :param path_resolver: Path resolver for determining file system locations.
        """

        self._path_resolver = path_resolver

    def create_config(self, app_path: str, user_inputs: dict[str, str]) -> AppConfig:
        """
        Creates an AppConfig from an app path and user inputs.

        :param app_path: Dot-separated app path (e.g., 'app.human_resource.employee').
        :param user_inputs: Dictionary of user-provided configuration values.
        :return: Configured AppConfig instance ready for app generation.
        """

        components = app_path.split('.')
        app_name = user_inputs.get('app_name', components[-1])

        return AppConfig(
            app_name=app_name,
            app_path=app_path,
            components=components,
            destination=self._path_resolver.get_app_destination(components),
            template_destination=self._path_resolver.get_template_destination(components),
            user_inputs=user_inputs
        )



# Generated by Django 5.1.8 on 2025-11-13 20:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_file', '0001_initial'),
        ('django_spire_notification_email', '0002_emailnotification_bcc_emailnotification_cc_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailnotification',
            name='attachments',
            field=models.ManyToManyField(blank=True, related_name='attachments', related_query_name='attachment', to='django_spire_file.file'),
        ),
    ]



from __future__ import annotations

from django_spire.auth.controller.controller import BaseAuthController


class BaseKnowledgeAuthController(BaseAuthController):
    def can_access_all_collections(self):
        return self.request.user.has_perm(
            'django_spire_knowledge.can_access_all_collections'
        )

    def can_add(self):
        return self.request.user.has_perm('django_spire_knowledge.add_collection')

    def can_change(self):
        return self.request.user.has_perm('django_spire_knowledge.change_collection')

    def can_change_collection_groups(self):
        return self.request.user.has_perm(
            'django_spire_knowledge.can_change_collection_groups'
        )

    def can_delete(self):
        return self.request.user.has_perm('django_spire_knowledge.delete_collection')

    def can_view(self):
        return self.request.user.has_perm('django_spire_knowledge.view_collection')



from django.apps import AppConfig

from django_spire.utils import check_required_apps


class HelpDeskConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    label = 'django_spire_help_desk'
    name = 'django_spire.help_desk'
    MODEL_PERMISSIONS = (
        {
            'name': 'help_desk',
            'verbose_name': 'Help Desk',
            'model_class_path': 'django_spire.help_desk.models.HelpDeskTicket',
            'is_proxy_model': False,
        },
    )

    REQUIRED_APPS = ('django_spire_core',)
    URLPATTERNS_INCLUDE = 'django_spire.help_desk.urls'
    URLPATTERNS_NAMESPACE = 'help_desk'

    def ready(self) -> None:
        check_required_apps(self.label)



from django.urls import path

from django_spire.theme.views import page_views

app_name = 'page'

urlpatterns = [
    path('dashboard/', page_views.dashboard_view, name='dashboard'),
    path('colors/', page_views.colors_view, name='colors'),
    path('django_glue/', page_views.django_glue_view, name='django_glue'),
    path('example/', page_views.example_view, name='example'),
    path('typography/', page_views.typography_view, name='typography'),
    path('buttons/', page_views.buttons_view, name='buttons'),
    path('badges/', page_views.badges_view, name='badges'),
    path('borders/', page_views.borders_view, name='borders'),
]



from __future__ import annotations

from django import forms
from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html
from django.utils.http import urlencode

from django_spire.ai import models
from django_spire.ai.mixins import AiUsageAdminMixin
from django_spire.core.forms.widgets import JsonTreeWidget


@admin.register(models.AiUsage)
class AiUsageAdmin(AiUsageAdminMixin):
    list_display = (
        'recorded_date',
        'was_successful',
        'event_count',
        'token_usage',
        'run_time_seconds_formatted',
        'view_interactions_link',
        'view_successful_interactions_link',
        'view_failed_interactions_link'
    )
    search_fields = ('recorded_date',)
    ordering = ('-recorded_date',)

    def get_readonly_fields(self, request, obj=None) -> list[str]:
        return [field.name for field in self.model._meta.fields]

    def view_interactions_link(
            self,
            ai_usage: models.AiUsage,
            was_successful: bool | None = None,
    ) -> str:
        was_successful_filter = '' if was_successful is None else f'&was_successful__exact={int(was_successful)}'

        url = (
                reverse("admin:django_spire_ai_aiinteraction_changelist")
                + "?"
                + urlencode({"ai_usage__id": f"{ai_usage.id}"})
                + was_successful_filter
        )

        if was_successful is None:
            interactions_count = ai_usage.interactions.count()
        elif was_successful:
            interactions_count = ai_usage.interactions.filter(was_successful=True).count()
        else:
            interactions_count = ai_usage.interactions.filter(was_successful=False).count()

        return format_html(
            f'<a href="{url}">{interactions_count} Interactions</a>'
        )

    view_interactions_link.short_description = "All"

    def view_successful_interactions_link(
            self,
            ai_usage: models.AiUsage,
    ) -> str:
        return self.view_interactions_link(ai_usage, was_successful=True)

    view_successful_interactions_link.short_description = "Successful"

    def view_failed_interactions_link(
            self,
            ai_usage: models.AiUsage,
    ) -> str:
        return self.view_interactions_link(ai_usage, was_successful=False)

    view_failed_interactions_link.short_description = "Failed"


class AiInteractionModelForm(forms.ModelForm):
    interaction = forms.JSONField(widget=JsonTreeWidget)

    class Meta:
        model = models.AiInteraction
        fields = '__all__'


@admin.register(models.AiInteraction)
class AiInteractionAdmin(AiUsageAdminMixin):
    form = AiInteractionModelForm

    list_display = (
        'actor',
        'callable_name',
        'was_successful',
        'event_count',
        'token_usage',
        'run_time_seconds_formatted',
        'created_datetime',
    )
    list_filter = ('module_name', 'callable_name', 'was_successful')
    search_fields = ('actor', 'user_email', 'user_first_name', 'user_last_name', 'module_name', 'callable_name')
    ordering = ('-created_datetime',)

    readonly_fields = [
        'actor',
        'created_datetime',
        'module_name',
        'callable_name',
        'was_successful',
        'user',
        'user_email',
        'user_first_name',
        'user_last_name',
        'exception',
        'stack_trace',
    ]
    fields = [
        'actor',
        'user',
        'created_datetime',
        'module_name',
        'callable_name',
        'was_successful',
        'interaction',
        'user_email',
        'user_first_name',
        'user_last_name',
        'exception',
        'stack_trace',
    ]



from __future__ import annotations

from typing import TYPE_CHECKING

from dandy import Prompt

from django_spire.contrib.service import BaseDjangoModelService
from django_spire.core.tags.intelligence.tag_set_bot import TagSetBot

if TYPE_CHECKING:
    from django_spire.knowledge.collection.models import Collection


class CollectionTagService(BaseDjangoModelService['Collection']):
    obj: Collection

    def process_and_set_tags(self):
        collection_prompt = Prompt()

        collection_prompt.sub_heading(self.obj.name)
        collection_prompt.text(self.obj.description)

        tag_set = TagSetBot().process(
            content=collection_prompt
        )

        self.obj.set_tags_from_tag_set(
            tag_set=tag_set,
        )

    def get_aggregated_tag_set(self):
        tag_set = self.obj.tag_set

        for collection in self.obj.children.active():
            tag_set.update(collection.services.tag.get_aggregated_tag_set())

        for entry in self.obj.entries.active():
            tag_set.update(entry.tag_set)

        return tag_set

    def get_aggregated_tag_set_simplified(self):
        tag_set = self.obj.tag_set_simplified

        for collection in self.obj.children.active():
            tag_set.update(collection.services.tag.get_aggregated_tag_set())

        for entry in self.obj.entries.active():
            tag_set.update(entry.tag_set_simplified)

        return tag_set




from django.contrib.auth.models import User

from django_spire.contrib.seeding import DjangoModelSeeder


class UserSeeder(DjangoModelSeeder):
    model_class=User
    cache_seed = False

    fields={
        'id': 'exclude',
        'username': ('faker', 'name'),
        'first_name': ('faker', 'first_name'),
        'last_name': ('faker', 'last_name'),
        'email': ('faker', 'email'),
        'is_staff': ('faker', 'boolean'),
        'is_superuser': False,
        'is_active': True,
        'date_joined': ('custom', 'date_time_between', {'start_date': '-30d', 'end_date': 'now'}),
        'password': 'exclude', # password hash slows down seeding
        'last_login': 'exclude'
    }
    default_to = 'faker'



from __future__ import annotations

from django.db.models import QuerySet


def filter_by_lookup_map(
        queryset: QuerySet,
        lookup_map: dict,
        data: dict,
        extra_filters: dict | None = None
):
    """
    Filters a given queryset based on a lookup map and provided data. Additional filters
    can also be applied if provided.

    Parameters:
        queryset (QuerySet): The queryset to be filtered.
        lookup_map (dict): A dictionary mapping input data keys to filtering fields in
            the queryset.
        data (dict): A dictionary containing key-value pairs to filter the queryset.
        extra_filters (list | None): Optional. A list of extra positional filter
            arguments to be applied. Defaults to None.

    Returns:
        QuerySet: The filtered queryset.
    """
    if extra_filters is None:
        extra_filters = {}

    lookup_kwargs = {
        lookup_map[k]: v
        for k, v in data.items()
        if k in lookup_map and v not in (None, "", [])
    } | extra_filters

    return queryset.filter(**lookup_kwargs)



from __future__ import annotations

from dandy import Bot, Prompt

from django_spire.knowledge.entry.models import Entry
from django_spire.knowledge.intelligence.intel.collection_intel import CollectionIntel
from django_spire.knowledge.intelligence.intel.entry_intel import EntryIntel


class EntrySearchBot(Bot):
    llm_role = 'Knowledge Entry Search Assistant'
    llm_task = 'Read through the knowledge entry the information request and return information and the block id that relevant'
    llm_guidelines = (
        Prompt()
        .list([
            'Make sure the relevant heading text is from a heading with mark down formatting.'
            'When returning the relevant heading remove any of the markdown formating characters.'
        ])
    )
    llm_intel_class = EntryIntel

    def process(self, user_input: str, entry: Entry) -> EntryIntel:
        entry_prompt = Prompt()
        entry_prompt.sub_heading('Information Request')
        entry_prompt.line_break()
        entry_prompt.text(f'{user_input}')
        entry_prompt.line_break()
        entry_prompt.sub_heading(f'Knowledge Entry: {entry.name}')
        entry_prompt.line_break()

        for version_block in entry.current_version.blocks.all():
            entry_prompt.text(f'{version_block.id}: {version_block.render_to_text()}')

        entry_intel = self.llm.prompt_to_intel(
            prompt=entry_prompt,
            exclude_fields={'entry_id', 'collection_intel'},
        )

        entry_intel.entry_id = entry.id
        entry_intel.collection_intel = CollectionIntel(
            name=entry.collection.name,
            collection_id=entry.collection.id,
        )

        return entry_intel



# Generated by Django 5.2.7 on 2025-10-22 01:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_knowledge', '0004_alter_collection_options_collectiongroup'),
    ]

    operations = [
        migrations.AddField(
            model_name='entryversionblock',
            name='_tunes_data',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='entryversionblock',
            name='type',
            field=models.CharField(choices=[('text', 'Text'), ('heading', 'Heading'), ('sub_heading', 'Sub Heading'), ('list_item', 'List Item'), ('list', 'List'), ('code', 'Code')], default='text', max_length=32),
        ),
    ]



# Generated by Django 5.2.7 on 2025-11-12 16:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_core', '0001_initial'),
        ('django_spire_knowledge', '0007_alter_collection_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='tags',
            field=models.ManyToManyField(blank=True, editable=False, null=True, related_name='+', to='django_spire_core.tag'),
        ),
        migrations.AddField(
            model_name='entry',
            name='tags',
            field=models.ManyToManyField(blank=True, editable=False, null=True, related_name='+', to='django_spire_core.tag'),
        ),
    ]



from dandy import Bot, Prompt, BaseListIntel
from django.utils.text import slugify


class TagsIntel(BaseListIntel[list[str]]):
    tags: list[str]


class TagSetBot(Bot):
    llm_role = 'Tag Identifier'
    llm_task = 'Read through the provided content and return a list of tags.'
    llm_guidelines = (
        Prompt()
        .list([
            'Make sure to have enough tags to properly cover all the provided content.',
            'Focus on tagging the words in the content',
            'Only add additional words that are very relevant to the content.'
            'Use spaces to separate words in tags',
            'Include known acronyms along with the full tags.',
        ])
    )

    def process(self, content: Prompt | str) -> set[str]:
        tags_intel = self.llm.prompt_to_intel(
            prompt=(
                Prompt()
                .heading('Content to be Tagged:')
                .line_break()
                .text(content)
            ),
            intel_class=TagsIntel
        )

        for tag in tags_intel:
            word_segments = tag.split(" ")

            if len(word_segments) > 1:
                for word_segment in word_segments:
                    tags_intel.append(word_segment)

        return {slugify(tag) for tag in tags_intel}



from __future__ import annotations

from django_spire.ai.context import models
from django_spire.ai.context import choices

from faker import Faker


models.Organization.objects.create(
    name='Spire Candy',
    legal_name='Spire Candy Processors Limited',
)

for people_role_choice in choices.PersonRoleChoices:
    first_name = Faker().first_name()
    last_name = Faker().last_name()

    people = models.Person.objects.create(
        role=people_role_choice.value,
        first_name=first_name,
        last_name=last_name,
        email=f'{first_name.lower()}.{last_name.lower()}@spirecandy.ca',
        phone=Faker().phone_number(),
    )



def get_matching_tag_set_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> set[str]:
    return tag_set_a.intersection(tag_set_b)


def get_matching_count_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> int:
    return len(get_matching_tag_set_from_tag_sets(tag_set_a, tag_set_b))


def get_matching_a_percentage_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> float:
    try:
        return get_matching_count_from_tag_sets(tag_set_a, tag_set_b) / len(tag_set_a)
    except ZeroDivisionError:
        return 0.0


def get_matching_b_percentage_from_tag_sets(tag_set_a: set[str], tag_set_b: set[str]) -> float:
    try:
        return get_matching_count_from_tag_sets(tag_set_a, tag_set_b) / len(tag_set_b)
    except ZeroDivisionError:
        return 0.0
